"""
计算各种文件哈希值

Calculate the hash value of various files
"""
import hashlib
import os

BlockSize = 8096


def hashWrapper(algorithm):
    """
    计算文件哈希值(通用)

    :param algorithm: 算法名 [md5, sha1, sha256, sha512]
    :return:
    """
    def Wrapper(func):
        def wrapper(filePath: str) -> str:
            global cal
            if not os.path.exists(filePath):
                return 'No such file: ' + filePath
            if not os.path.isfile(filePath):
                return 'Not a file: ' + filePath
            cal = func()
            with open(filePath, 'rb') as f:
                while True:
                    data = f.read(BlockSize)
                    if not data:
                        break
                    cal.update(data)
            return cal.hexdigest()
        return wrapper
    return Wrapper


@hashWrapper('md5')
def md5():
    return hashlib.md5()


@hashWrapper('sha1')
def sha1():
    return hashlib.sha1()


@hashWrapper('sha256')
def sha256():
    return hashlib.sha256()


@hashWrapper('sha512')
def sha512():
    return hashlib.sha512()



# coding=utf-8
"""
阿里云相关API

Alibaba cloud API
"""
from . import pre_check, user_lang, dir_char
from .. import requirePackage
oss2 = requirePackage('oss2')


class AliyunOSS:
    def __init__(self, ac_id=pre_check('aliyun_oss_acid'), ac_key=pre_check('aliyun_oss_ackey'),
                 bucket_url=pre_check('aliyun_oss_df_endpoint'), df_bucket=pre_check('aliyun_oss_df_bucket')):
        """
        初始化并登陆阿里云对象存储

        Initializes and logs into ali cloud object storage

        :param ac_id: Access id
        :param ac_key: Access key
        :param bucket_url: oss-[地区 | location].aliyuncs.com
        :param df_bucket: 桶名称
        """
        self.ac_id = ac_id
        self.ac_key = ac_key
        self.bucket_url = bucket_url
        self.df_bucket = df_bucket
        self.progress, self.trans_id, self._progress_file_name = None, None, None
        self.auth = oss2.Auth(self.ac_id, self.ac_key)

    def get_func_table(self):
        """
        获取OSS类支持的操作

        Get the operations supported by the OSS class

        :return: 支持的函数字典
        """
        return {
            '-up': self.upload,
            '-rm': self.remove,
            '-dl': self.download,
            '-ls': self.list_bucket
        }

    def _progress_bar(self, cur, total):
        """
        创建一个进度条并启动

        Create a progress bar and start

        :param cur: 当前完成量
        :param total: 总任务量
        :return: None
        """
        if not self.progress:
            from ..TuiTools.Bar import DataTransformBar
            self.progress = DataTransformBar(True if total else False)
            self.trans_id = self.progress.add_task('Transform', total=total if total else -1,
                                                   filename=self._progress_file_name)
            self.progress.start()
            self.progress.start_task(self.trans_id)
        self.progress.update(self.trans_id, completed=cur)

    def upload(self, filePath: str, bucket: str = None):
        """
        上传文件（支持断点续传）

        Upload file (support breakpoint continuation)

        :param filePath: 文件路径
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        import os
        from .. import qs_default_console, qs_info_string
        from ..SystemTools import get_core_num
        bucket = bucket if bucket else self.df_bucket
        bucket = oss2.Bucket(self.auth, self.bucket_url, bucket)
        filePath = filePath.strip()
        self._progress_file_name = os.path.basename(filePath)
        oss2.resumable_upload(bucket, filePath.replace(dir_char, '/'), filePath, num_threads=get_core_num() * 4,
                              progress_callback=self._progress_bar)
        self.progress.stop()
        qs_default_console.print(qs_info_string, 'Transform Completed!' if user_lang != 'zh' else '传输完成!')

    def download(self, filename: str, bucket: str = None):
        """
        下载文件（支持断点续传）

        Download file (support breakpoint continuation)

        :param filename: 文件在bucket中的路径
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        import os
        from .. import qs_default_console, qs_info_string
        from ..SystemTools import get_core_num
        bucket = bucket if bucket else self.df_bucket
        bucket = oss2.Bucket(self.auth, self.bucket_url, bucket)
        self._progress_file_name = os.path.basename(filename)
        oss2.resumable_download(bucket, filename, filename, num_threads=get_core_num() * 4,
                                progress_callback=self._progress_bar)
        self.progress.stop()
        qs_default_console.print(qs_info_string, 'Transform Completed!' if user_lang != 'zh' else '传输完成!')

    def remove(self, filePath: str, bucket: str = None):
        """
        删除文件

        delete file

        :param filePath: 文件在bucket中的路径
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        bucket = bucket if bucket else self.df_bucket
        bucket = oss2.Bucket(self.auth, self.bucket_url, bucket)
        bucket.delete_object(filePath)

    def list_bucket(self, bucket: str = None):
        """
        打印桶中的文件表

        Print the file table in the bucket

        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        from .. import qs_default_console, qs_info_string
        from ..NetTools.NormalDL import size_format
        from ..TuiTools.Table import qs_default_table
        from rich.text import Text

        bucket = bucket if bucket else self.df_bucket
        ls = oss2.Bucket(self.auth, self.bucket_url, bucket)
        tb = qs_default_table(['File', 'Size'] if user_lang != 'zh' else ['文件', '体积'], title='Aliyun OSS')
        prefix = dict()
        for obj in oss2.ObjectIterator(ls):
            if '/' in obj.key:
                prefix[obj.key[obj.key[:obj.key.index('/')]]] = 0
            tb.add_row(Text(obj.key, justify='left'), Text(size_format(obj.size, True), justify='right'))
        qs_default_console.print(
            qs_info_string,
            'Bucket Url:' if user_lang != 'zh' else '桶链接:',
            'https://' + bucket + '.' + self.bucket_url + '/'
        )
        qs_default_console.print(tb, justify="center")



# coding=utf-8
"""
修改图像颜色工具

Modify image color tool
"""
from PIL import Image


def transport_back(src: str, to_color: tuple, from_color: tuple = (0, 0, 0, 0)) -> Image.Image:
    """
    图片颜色替换

    Image color replacement

    :param src: 图片路径
    :param to_color: RGBA四元组 -> 转换至目标颜色
    :param from_color: RGBA四元组 -> 需要被转换的颜色
    :return:
    """
    src = Image.open(src)
    src = src.convert('RGBA')
    L, H = src.size
    color_0 = from_color
    transparency_flag = False if color_0[-1] else True
    for h_indx in range(H):
        for l_indx in range(L):
            dot = (l_indx, h_indx)
            color_1 = src.getpixel(dot)
            if transparency_flag and not color_1[-1]:
                src.putpixel(dot, to_color)
            elif color_0[:-1] == color_1[:-1] and color_1[-1]:
                src.putpixel(dot, to_color)
    return src


def formatOneColor(src: str, to_color: tuple, except_color=None) -> Image.Image:
    """
    格式化图片颜色

    Format image color

    :param src: 图片路径
    :param to_color: RGBA四元组 -> 转换至目标颜色
    :param except_color: RGBA四元组 -> 忽略的颜色
    :return:
    """
    if except_color is None:
        except_color = [(0, 0, 0, 0)]
    src = Image.open(src)
    src = src.convert('RGBA')
    L, H = src.size
    for h_indx in range(H):
        for l_indx in range(L):
            color_1 = src.getpixel(dot := (l_indx, h_indx))
            if color_1 not in except_color:
                src.putpixel(dot, to_color)
    return src


def get_color_from_str(str_color: str) -> tuple:
    """
    解析字符串为RGBA四元组

    Parse strings for RGBA quaternions

    :param str_color: 表示颜色的字符串(支持16进制、RGB或RGBA) | String representing color (hex, RGB, or RGBA support)
    :return: RGBA四元组 | RGBA quad
    """
    if ',' in str_color:
        str_color = [int(i) for i in str_color.split(',')]
        if len(str_color) == 3:
            str_color.append(255)
        str_color = tuple(str_color)
    elif len(str_color) == 6:
        str_color = (int(str_color[:2], 16), int(str_color[2:4], 16), int(str_color[4:], 16), 255)
    else:
        exit('ERROR COLOR!')
    return str_color



# coding=utf-8
"""
调用各种系统工具

Call various system tools
"""
import QuickStart_Rhy.Wrapper as _wrapper

miss_file = ['.DS_Store']


def __latest_filename(name):
    import os
    from . import dir_char

    cur = os.getcwd()
    rec = cur
    while cur != dir_char:
        if os.path.exists(name):
            os.chdir(rec)
            return cur + dir_char + name
        os.chdir('..')
        cur = os.getcwd()
    os.chdir(rec)
    return ''


def top():
    """
    CPU和内存监测

    CPU and memory monitoring

    :return: None
    """
    from . import dir_char
    if dir_char == '\\':
        from .SystemTools.Monitor import top
        top()
    else:
        import sys
        sys.argv = ['bpytop'] + sys.argv[2:]

        from . import requirePackage
        requirePackage('bpytop', 'main')()


def clear_mem():
    """
    清理系统内存

    Clean system memory

    :return: None
    """
    from .SystemTools import clear_mem
    clear_mem()


def go_github():
    """
    自动识别当前文件夹.git/config中的地址，并通过浏览器打开

    Automatically recognize the address in the current folder .git/config and open it through a browser
    """
    import os
    from . import qs_default_console, qs_error_string, user_lang, open_url, requirePackage

    config_path = __latest_filename('.git/config')
    if not os.path.exists(config_path):
        qs_default_console.print(qs_error_string, 'No a git dictionary' if user_lang != 'zh' else '不是 git 文件夹')
        return
    config = requirePackage('configparser', 'ConfigParser')()
    config.read(filenames=config_path)
    url_ls = []
    for section in config.sections():
        if section.startswith('remote'):
            url_ls.append(config[section]['url'].replace('.git', ''))
    open_url(url_ls)


@_wrapper.mkCompressPackageWrap
def _mktar(file_path: str = ''):
    from .SystemTools.Compress import Tar
    return Tar(file_path + '.tar.gz', 'w')


def mktar():
    """
    创建tar包

    Create a tar packages

    :return: None
    """
    return _mktar()


@_wrapper.unCompressPackageWrap
def _untar(file_path: str = ''):
    from .SystemTools.Compress import Tar
    return Tar(file_path)


def untar():
    """
    解压tar包

    Unpack the tar packages

    :return: None
    """
    return _untar()


@_wrapper.mkCompressPackageWrap
def _mkzip(file_path: str = ''):
    from .SystemTools.Compress import Zip
    return Zip(file_path + '.zip', 'w')


def mkzip():
    """
    创建ZIP包

    Create a ZIP package

    :return: None
    """
    return _mkzip()


@_wrapper.unCompressPackageWrap
def _unzip(file_path: str = ''):
    from .SystemTools.Compress import Zip
    return Zip(file_path, 'r')


def unzip():
    """
    解压ZIP包

    Unpack the ZIP package

    :return: None
    """
    return _unzip()


@_wrapper.unCompressPackageWrap
def _unrar(file_path: str = ''):
    from .SystemTools.Compress import Rar
    return Rar(file_path)


def unrar():
    """
    解压RAR包

    Extract RAR package

    :return: None
    """
    return _unrar()


@_wrapper.mkCompressPackageWrap
def _mk7z(file_path: str = ''):
    from .SystemTools.Compress import SevenZip
    return SevenZip(file_path + '.7z', 'w')


def mk7z():
    """
    创建7z包

    Create 7z package

    :return: None
    """
    return _mk7z()


@_wrapper.unCompressPackageWrap
def _un7z(file_path: str = ''):
    from .SystemTools.Compress import SevenZip
    return SevenZip(file_path)


def un7z():
    """
    解压7z包

    Extract 7z package

    :return:
    """
    return _un7z()


@_wrapper.HashWrapper('md5')
def md5():
    """
    获取文件md5值
    :return:
    """


@_wrapper.HashWrapper('sha1')
def sha1():
    """
    获取文件sha1值
    :return:
    """


@_wrapper.HashWrapper('sha256')
def sha256():
    """
    获取文件sha256值
    :return:
    """


@_wrapper.HashWrapper('sha512')
def sha512():
    """
    获取文件sha512值
    :return:
    """


def diff_dir():
    """
    对比两个文件夹差异，并生成相应html对比结果
    :return:
    """
    from . import user_lang, qs_default_console, qs_info_string
    from .SystemTools.Diff import DictionaryFiles
    import sys

    if '-h' in sys.argv:
        qs_default_console.print(qs_info_string, 'Usage: qs diff <dir1> <dir2> [-x <name or regex pattern>]')

    d1, d2 = sys.argv[2:4]
    apply_ignore = sys.argv[sys.argv.index('-x') + 1:] if '-x' in sys.argv else None
    d1 = DictionaryFiles(d1, apply_ignore)
    d2 = DictionaryFiles(d2, apply_ignore)

    if not (d1.available and d2.available):
        return

    with qs_default_console.status('Generating diff result..' if user_lang != 'zh' else '生成对比结果中..'):
        from .SystemTools.Diff import DiffFilesToStructHtml

        DiffFilesToStructHtml(d1, d2).generate()



# coding=utf-8
"""
命令入口

Command entry
"""
import sys
from .funcList import *
from . import requirePackage


def qs_help(rep=''):
    """输出菜单 | Output menu"""
    if not rep or rep not in menu_table:
        if user_lang != 'zh':
            color_rep("""
    qs basic   :-> basic   tools help | u, a, f, cal, time, pcat..
    qs system  :-> system  tools help | top, \[mk/un]\[zip/tar/7z]..
    qs net     :-> network tools help | http, dl, netinfo, wifi...
    qs api     :-> api     tools help | trans, smms, rmbg, loli...
    qs image   :-> image   tools help | stbg, v2gif, v2mp3, v2mp4.""", True)
            qs_default_console.print('\n[underline]Tutorial[/underline]\n', justify='center')
            qs_default_console.print('[bold magenta]qs lesson[/bold magenta]', justify='center')
            qs_default_console.print('\n[underline]Docs\n', justify='center')
            qs_default_console.print('C D N : https://rhythmlian.cn/2020/02/14/QuickStart-Rhy/ _ _ _', justify='center')
            qs_default_console.print('Github: https://rhythmicc.github.io/2020/02/14/QuickStart-Rhy/', justify='center')
            qs_default_console.print('\n[underline]TG Group:\n', justify='center')
            qs_default_console.print('https://t.me/joinchat/G2mpk7-S85eM7sb7', justify='center')
        else:
            color_rep("""
    qs basic   :-> 基础工具帮助 | u, a, f, cal, time, pcat..
    qs system  :-> 系统工具帮助 | top, \[mk/un]\[zip/tar/7z]..
    qs net     :-> 网络工具帮助 | http, dl, netinfo, wifi...
    qs api     :-> 扩展工具帮助 | trans, smms, rmbg, loli...
    qs image   :-> 图像工具帮助 | stbg, v2gif, v2mp3, v2mp4.""", True)
            qs_default_console.print('\n[underline]引导[/underline]\n', justify='center')
            qs_default_console.print('[bold magenta]qs lesson[/bold magenta]', justify='center')
            qs_default_console.print('\n[underline]文档[/underline]\n', justify='center')
            qs_default_console.print('C D N : https://rhythmlian.cn/2020/08/09/QuickStart-Rhy-zh/ _ _ _', justify='center')
            qs_default_console.print('Github: https://rhythmicc.github.io/2020/08/09/QuickStart-Rhy-zh/', justify='center')
            qs_default_console.print('\n[underline]TG群[/underline]\n', justify='center')
            qs_default_console.print('https://t.me/joinchat/G2mpk7-S85eM7sb7', justify='center')
    else:
        menu_table[rep]()


def main():
    """执行命令 | Execute"""
    debug_flag = '--qs-debug' in sys.argv
    if debug_flag:
        sys.argv.remove('--qs-debug')
    if len(sys.argv) >= 2:
        try:
            func_name = sys.argv[1]
            cmd_config = {j: i for i in [basic_funcs, api_funcs, net_funcs, image_funcs, system_funcs] for j in i}
            cmd_config.pop('self')
            if func_name not in cmd_config:
                qs_help(func_name)
            else:
                func_table = cmd_config[func_name]
                file_name = func_table['self']
                func_name = func_table[func_name]
                (
                    requirePackage('QuickStart_Rhy', func_name)
                    if file_name == 'basic' else
                    requirePackage(f'QuickStart_Rhy.{file_name}', func_name)
                )()
        except Exception as e:
            from . import qs_default_console
            if debug_flag:
                qs_default_console.print_exception()
            else:
                from . import qs_error_string
                qs_default_console.log(qs_error_string, repr(e))
    else:
        qs_help()


if __name__ == '__main__':
    main()



"""
qs的WIFI模块, 协助连接WIFI, 但不支持机构WIFI自动登录 | 仅在Mac OS下工作

The WiFi module of QS helps to connect WiFi, but does not support the automatic login of institutional WiFi |
(Works only on Mac OS)
"""
import os
import re
from .. import qs_default_console, qs_warning_string, qs_info_string


class WiFi:
    def __init__(self) -> None:
        """
        适用于Mac OS X的wifi工具

        Wifi tools for Mac OS X
        """
        with os.popen('networksetup -listallhardwareports') as pipe:
            ifaces = pipe.read()
        ifaces = [i.split('\n') for i in ifaces.strip().split('\n\n')]
        ifaces.remove(['VLAN Configurations', '==================='])
        self.ifaces = ifaces
        for i in self.ifaces:
            if i[0].endswith('Wi-Fi'):
                self.iface = [j.split()[-1] for j in i[1:]]
                return

    def status(self) -> str:
        """
        判断当前wifi连接状态

        Determine the current wifi connection status

        :return: 连接的wifi名 | The name of the connecting wifi
        """
        with os.popen('networksetup -getairportnetwork %s' % self.iface[0]) as pipe:
            res = pipe.read().strip()
            res = re.sub('.*?:', '', res).strip()
        qs_default_console.print(qs_info_string, f"{'已' if res else '未'}连接", res if res else '')
        return res

    @staticmethod
    def scan():
        """
        扫描附近可连接的wifi

        Scan for nearby wifi connections

        :return: 按信号强度排序好的可连接wifi列表 | A list of available wifi connections sorted by signal strength
        """
        from . import is_mac
        with os.popen(
                '/System/Library/PrivateFrameworks/Apple80211.framework/Versions/A/Resources/airport scan') as pipe:
            res = pipe.read().strip().split('\n')[1:]
            tmp_ls = [i.strip().split() for i in res]
            has_add = set()
            res = []
            for i in tmp_ls:
                mac_index = 0
                while mac_index < len(i):
                    if is_mac(i[mac_index]):
                        break
                    else:
                        mac_index += 1
                if mac_index >= len(i):
                    qs_default_console.log(qs_warning_string, "Failed to get info with:", i)
                    continue
                ssid = ' '.join(i[:mac_index])
                if i[0] not in has_add:
                    res.append([ssid, int(i[mac_index+1]), i[-1]])
                    has_add.add(ssid)
        return sorted(res, key=lambda x: x[1], reverse=True)

    def set_iface(self):
        raise NotImplementedError

    def conn(self, ssid: list, password: str):
        """
        连接WiFi

        connect WiFi

        :param ssid: wifi名称
        :param password: 密码
        :return: status
        """
        os.system('networksetup -setairportnetwork %s %s "%s"' % (self.iface[0], ssid[0], password))
        return self.status()

    def disconn(self):
        return NotImplementedError



from .. import dir_char, qs_default_console, qs_error_string, qs_warning_string, user_lang
from ..SystemTools.FileHash import md5
import os
import re


class DictionaryFiles(set):
    def __init__(self, rt: str, ignorePatterns: list = None):
        """
        :param rt: 目录路径
        """
        super().__init__()
        if ignorePatterns is None:
            ignorePatterns = []
        if not (os.path.exists(rt) or os.path.isdir(rt)):
            qs_default_console.log(
                qs_error_string,
                f'"{rt}" {"Not exist or Not a dictionary!" if user_lang != "zh" else "不存在或不是文件夹"}'
            )
            self.available = False
            return
        self.name = rt.strip(dir_char).split(dir_char)[-1]
        self.rt = os.path.abspath(rt)
        self.available = True
        self.ignored = set(ignorePatterns) if ignorePatterns else None
        self.__getAllPaths()

    def checkIgnore(self, path):
        if self.ignored:
            for item in self.ignored:
                if re.findall(item, path):
                    return True
        return False

    def __getAllPaths(self):
        for rt, sonDir, files in os.walk(self.rt):
            for file in files:
                filePath = os.path.join(rt, file).replace(self.rt, '')
                if self.checkIgnore(filePath):
                    continue
                self.add(filePath)

    def getMd5ByFileItem(self, item: str):
        return md5(self.rt + item)

    def getFilepathByItem(self, item: str):
        return self.rt + item


class DiffFilesToStructHtml:
    from concurrent.futures import ThreadPoolExecutor, wait
    from rich.progress import Progress
    from difflib import HtmlDiff

    def __init__(self, d1: DictionaryFiles, d2: DictionaryFiles):
        self.d1 = d1
        self.d2 = d2
        self.rt = os.getcwd() + dir_char + d1.name + '-vs-' + d2.name + '.qs_diff'
        self.pool = DiffFilesToStructHtml.ThreadPoolExecutor(max_workers=8)
        self.jobLs = []
        self.progress = DiffFilesToStructHtml.Progress(console=qs_default_console)
        self.pid = self.progress.add_task('compare' if user_lang != 'zh' else '对比')

    def _run(self, item: str):
        pathLs = item.strip(dir_char).split(dir_char)[:-1]
        for i in range(len(pathLs)):
            dirName = self.rt + dir_char + dir_char.join(pathLs[:i+1])
            if not os.path.exists(dirName):
                os.mkdir(dirName)
        with open(self.d1.getFilepathByItem(item), 'r') as f:
            _d1 = f.readlines()
        with open(self.d2.getFilepathByItem(item), 'r') as f:
            _d2 = f.readlines()
        _diff = DiffFilesToStructHtml.HtmlDiff().make_file(_d1, _d2)
        with open(self.rt + item + '.html', 'w') as f:
            f.write(_diff)
        self.progress.advance(self.pid, 1)

    def generate(self):
        if os.path.exists(self.rt):
            qs_default_console.log(
                qs_warning_string,
                f'"{self.rt}" {"Already exists! QS will delete it and regenerate." if user_lang != "zh" else "已经存在! QS将删除它并重新生成."}'
            )
            from .. import remove
            remove(self.rt)

        os.mkdir(self.rt)
        _tmpLs = []
        for item in list(self.d1 & self.d2):
            if self.d1.getMd5ByFileItem(item) != self.d2.getMd5ByFileItem(item):
                _tmpLs.append(item)
        self.progress.update(self.pid, total=len(_tmpLs))
        self.progress.start()
        self.progress.start_task(self.pid)

        for item in _tmpLs:
            self.jobLs.append(self.pool.submit(self._run, item))

        DiffFilesToStructHtml.wait(self.jobLs)
        self.progress.stop()

        with open(self.rt + dir_char + 'README.md', 'w') as f:
            print("# Diff Results | 目录对比结果", file=f, end='\n\n')

            print(
                "## Several documents with differences as shown in the table below"
                if user_lang != 'zh' else '## 存在若干有差异的文件如下表',
                file=f,
                end='\n\n'
            )

            print(f'|{"Path" if user_lang != "zh" else "路径"}|{"Results Link" if user_lang != "zh" else "结果链接"}|\n|---|:---:|', file=f)
            for item in sorted(_tmpLs):
                print(f'|{item}|[{"result" if user_lang != "zh" else "结果"}](file://{self.rt + item + ".html"})|', file=f)

            print(
                "## There are several non-shared documents as shown in the table below"
                if user_lang != 'zh' else '## 存在若干非共有文件如下表',
                file=f,
                end='\n\n'
            )

            print(f'|{self.d1.name}|{self.d2.name}|\n|---|---|', file=f)
            for item in sorted(list(self.d1 ^ self.d2)):
                print(f'|{item if item in self.d1 else " "}|{item if item in self.d2 else " "}|', file=f)



# coding=utf-8
"""
qs用到的函数装饰器, 在使用它们前, 请确保你理解注释描述的内容

Before using function decorators used by QS, please make sure that you understand the content described by the annotation.
"""


def set_timeout(num: int):
    """
    定时函数装饰器

    Timing function decorator

    :param num: 时间（秒）
    :return: wrapper
    """
    def wrapper(func):
        def handle(signum, frame):
            raise RuntimeError

        def run(*args, **kwargs):
            import signal
            try:
                signal.signal(signal.SIGALRM, handle)
                signal.alarm(num)
                res = func(*args, **kwargs)
                signal.alarm(0)
                return res
            except RuntimeError:
                return False
        return run
    return wrapper


def mkCompressPackageWrap(func):
    """
    创建压缩文件的通用函数装饰器, 被装饰的函数将被传递压缩文件名, 你需要返回:
        QuickStart_Rhy.SystemTools.Compress._NormalCompressedPackage
    的子类对象, 并将读写状态设为 写 状态

    General function decorator for creating compressed files, the decorated function will be passed the compressed file
    name, which you need to return:
        QuickStart_Rhy.SystemTools.Compress._NormalCompressedPackage
    's sub object, and set read-write mode as write

    :param func: func(filePath: str = '') -> QuickStart_Rhy.SystemTools.Compress._NormalCompressedPackage
    :return: 装饰器 | wrapper
    """
    def wrapper():
        import os
        from .. import dir_char
        from ..SystemTools.Compress import get_compress_package_name
        packages_name, ls = get_compress_package_name()
        packages = func(packages_name)

        def dfs(cur_p):
            if os.path.isfile(cur_p):
                packages.add_file(cur_p)
                return
            file_ls = os.listdir(cur_p)
            flag_ch = '' if cur_p.endswith(dir_char) else dir_char
            for fp in file_ls:
                fp = cur_p + flag_ch + fp
                dfs(fp)

        for i in ls:
            dfs(i)
        packages.save()
    return wrapper


def unCompressPackageWrap(func):
    """
    解压缩文件的通用函数装饰器, 被装饰的函数将被传递压缩文件名, 你需要返回:
        QuickStart_Rhy.SystemTools.Compress._NormalCompressedPackage
    的子类对象, 并将读写状态设为 读 状态

    General function decorator for extracting compressed files, the decorated function will be passed the compressed file
    name, which you need to return:
        QuickStart_Rhy.SystemTools.Compress._NormalCompressedPackage
    's sub object, and set read-write mode as read

    :param func: func(filePath: str = '') -> QuickStart_Rhy.SystemTools.Compress._NormalCompressedPackage
    :return: 装饰器 | wrapper
    """
    def wrapper():
        import os
        import sys

        file_names = sys.argv[2:]
        if not file_names:
            exit("No enough parameters")
        from .. import qs_default_console, qs_error_string
        from ..NetTools.NormalDL import core_num
        from ..ThreadTools import ThreadPoolExecutor, wait
        pool = ThreadPoolExecutor(max_workers=max(core_num // 2, 4))
        job_q = []

        def run(path):
            if os.path.exists(path):
                try:
                    cur_tar = func(path)
                    cur_tar.extract()
                except Exception as e:
                    qs_default_console.log(qs_error_string, repr(e))
            else:
                qs_default_console.log(qs_error_string, "No such file or dictionary: %s" % path)

        for file_name in file_names:
            job_q.append(pool.submit(run, file_name))
        wait(job_q)
    return wrapper


def HashWrapper(algorithm: str):
    """
    文件哈希值计算(通用函数)

    :param algorithm: 算法名称 [md5, sha1, sha256, sha512]
    :return:
    """
    def Wrapper(func):
        def _wrapper():
            import sys, os
            from .. import qs_default_console, qs_info_string, user_lang

            ls = sys.argv[2:]
            if not ls:
                qs_default_console.print(qs_info_string, 'Usage: qs %s file1 file2 ...' % algorithm)
                return

            exec('from QuickStart_Rhy.SystemTools.FileHash import %s' % algorithm)
            for file in ls:
                qs_default_console.print(
                    qs_info_string, 'Calculate:' if user_lang != 'zh' else '计算:', os.path.basename(file)
                )
                exec(f'qs_default_console.print(qs_info_string, "{algorithm}" + ":", {algorithm}("{file}"))')
        return _wrapper
    return Wrapper



# coding=utf-8
"""
m3u8文件的下载器

The downloader of m3u8 file
"""
import urllib3
from concurrent.futures import ThreadPoolExecutor, wait
import requests
import os
import queue
from .. import dir_char, remove, headers, user_lang, qs_default_console, qs_error_string, qs_info_string
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def merge_file(path, ts_ls, name):
    """
    将全部ts文件合并

    Merge all TS files

    :param path: 文件夹路径
    :param ts_ls: ts文件路径列表
    :param name: 合并后的ts文件名
    :return:
    """
    if not path.endswith(dir_char):
        path += dir_char
    with open(name + '.ts', 'wb') as f:
        for ts in ts_ls:
            with open(path + ts, 'rb') as ff:
                f.write(ff.read())


class M3U8DL:
    from rich.progress import Progress, TimeRemainingColumn, TextColumn, BarColumn

    proxies = {}

    def __init__(self, target, name, proxy: str = ''):
        """
        初始化M3U8下载引擎

        Initialize the M3U8 download engine

        :param target: 目标url
        :param name: 文件名
        """
        self.path = ''
        self._cur = 0
        self._all = 0
        self.target = target
        self.name = name
        self.job_queue = queue.Queue()
        if proxy:
            M3U8DL.proxies = {
                'http': 'http://'+proxy,
                'https': 'https://'+proxy
            }
        self.main_progress = M3U8DL.Progress(
            M3U8DL.TextColumn("[bold blue]{task.fields[taskName]}", justify="right"),
            M3U8DL.BarColumn(bar_width=None),
            "[progress.percentage]{task.percentage:>3.1f}%",
            "•",
            M3U8DL.TimeRemainingColumn(),
            console=qs_default_console
        )
        self.dl_id = self.main_progress.add_task("Download", taskName='Downloading' if user_lang != 'zh' else '下载中')

    def _dl_one(self, job):
        """
        下载一个ts文件

        Download a TS file

        :param job: 任务信息
        :return: None
        """
        try:
            if job[-1]:
                pd_url = job[0]
                c_fule_name = job[1]
                if not os.path.exists(os.path.join(self.path, c_fule_name)):
                    res = requests.get(pd_url, verify=False, proxies=M3U8DL.proxies)
                    with open(os.path.join(self.path, c_fule_name), 'ab') as f:
                        f.write(res.content)
                        f.flush()
            self.main_progress.advance(self.dl_id, 1)
        except Exception as e:
            qs_default_console.log(qs_error_string, repr(e))
            self.job_queue.put(job)

    def download(self):
        """
        下载

        :return: None
        """
        target = self.target
        download_path = os.getcwd() + dir_char + self.name
        self.path = download_path
        if not os.path.exists(download_path):
            os.mkdir(download_path)
        try:
            all_content = requests.get(target, verify=False, headers=headers, proxies=M3U8DL.proxies).text
        except Exception as e:
            qs_default_console.log(qs_error_string, repr(e))
            return
        if "#EXTM3U" not in all_content:
            raise Exception("Not M3U8 Link" if user_lang != 'zh' else "非M3U8的链接")
        if "EXT-X-STREAM-INF" in all_content:
            file_line = all_content.split("\n")
            for line in file_line:
                if '.m3u8' in line:
                    target = target.rsplit("/", 1)[0] + "/" + line
                    all_content = requests.get(target, verify=False, headers=headers, proxies=M3U8DL.proxies).text
        file_line = all_content.split("\n")
        _rt = target.rsplit("/", 1)[0] + "/"
        tmp = []
        for index, line in enumerate(file_line):
            if "#EXT-X-KEY" in line:
                tmp.append((_rt, line, 0))
            if "EXTINF" in line:
                tmp.append((_rt + file_line[index + 1], file_line[index + 1].rsplit("/", 1)[-1], 1))
        file_line = tmp[::-1]
        self._all = len(file_line)
        tmp = [jb[1] for jb in file_line]

        self.main_progress.update(self.dl_id, total=len(file_line))
        for i in file_line:
            self.job_queue.put(i)
        pool = ThreadPoolExecutor(16)
        self.main_progress.start()
        self.main_progress.start_task(self.dl_id)
        while not self.job_queue.empty():
            cur_work = []
            while not self.job_queue.empty():
                cur_work.append(pool.submit(self._dl_one, self.job_queue.get()))
            wait(cur_work)
        self.main_progress.stop()

        qs_default_console.print(qs_info_string, "Download completed! Start merge.."
                                 if user_lang != 'zh' else '下载完成! 开始合并..')
        merge_file(download_path, tmp, self.name)
        remove(download_path)



# coding=utf-8
"""
调用各种qs的API

Call various QS API
"""
import sys
from . import user_lang, system, qs_default_console, qs_error_string, qs_info_string, qs_warning_string, requirePackage, force_show_img


def remove_bg():
    """
    删除图片背景

    Delete image background
    """
    try:
        path = sys.argv[2]
    except IndexError:
        qs_default_console.log(qs_error_string,
                               '%s: qs rmbg <%s>' % (('Usage', 'picture') if user_lang != 'zh' else ('用法', '图像')))
        return
    else:
        if path == '-helpF':
            qs_default_console.print(qs_info_string,
                                     '%s: qs rmbg <%s>' % (('Usage', 'picture')
                                                           if user_lang != 'zh' else ('用法', '图像')))
            return
        from .API.SimpleAPI import rmbg
        with qs_default_console.status('Dealing..' if user_lang != 'zh' else '处理中..') as st:
            rmbg(path)
            st.update(status='Done')


def smms():
    """
    上传图片或Markdown中图片到smms

    Upload images or Markdown images to SMMS
    """
    try:
        path = sys.argv[2]
    except IndexError:
        qs_default_console.log(
            qs_error_string,
            '%s: qs smms <%s>' % (('Usage', 'picture | *.md')
                                  if user_lang != 'zh' else ('用法', '图像 | *.md'))
        )
        return
    else:
        if path == '-help':
            qs_default_console.print(
                qs_info_string,
                '%s: qs smms <%s>' % (('Usage', 'picture | *.md')
                                      if user_lang != 'zh' else ('用法', '图像 | *.md'))
            )
            return
        from .API.SimpleAPI import smms
        with qs_default_console.status('Dealing..' if user_lang != 'zh' else '处理中..') as st:
            smms(path)
            st.update(status='Done')


def up_img():
    """
    上传图片或Markdown中图片到多平台（不保证数据安全）

    Upload images or Markdown images to multiple platforms (data security is not guaranteed)
    """

    def showSptPlatform(dt, ls):
        from rich.table import Table
        from rich.box import SIMPLE

        tb = Table('1', '2', '3', show_header=False, show_edge=False, box=SIMPLE)
        for indx in range(0, len(ls), 3):
            row = []
            for i in range(indx, indx + 3):
                if i < len(ls):
                    row.append(f'{ls[i]}: {dt[ls[i]]}')
                else:
                    break
            tb.add_row(*row)
        qs_default_console.print(
            qs_info_string,
            'Usage: qs upimg <picture | *.md> [platform]\n\nSupport ([platform]: description):'
            if user_lang != 'zh' else '用法: qs -upimg <图像 | *.md> [平台]\n\n支持 ([可选平台]: 描述):')
        qs_default_console.print(tb, justify="center")

    try:
        path = sys.argv[2]
    except IndexError:
        qs_default_console.log(qs_error_string, '%s: qs upimg <%s>' % (('Usage', 'picture | *.md')
                                                                       if user_lang != 'zh' else ('用法', '图像 | *.md')))
        return
    else:
        from .API.alapi import upload_image, _upimg_get_avaliable_platform
        import random

        spt_type = _upimg_get_avaliable_platform()
        spt_type_keys = list(spt_type.keys())
        if '-h' in sys.argv or '--help' in sys.argv:
            showSptPlatform(spt_type, spt_type_keys)
            qs_default_console.print(
                qs_info_string,
                'If you do not set platform, qs will randomly choose one.' if user_lang != 'zh' else
                '如果你没有设置平台，qs将随机抽取一个可用平台')
            return
        argv_len_3 = len(sys.argv) > 3
        if argv_len_3:
            sys.argv[3] = sys.argv[3].lower()
            type_map = {}
            for i in spt_type:
                type_map[i.lower()] = i

            upload_image(path, type_map[sys.argv[3]]) \
                if argv_len_3 and sys.argv[3] in type_map else (
                qs_default_console.print(
                    qs_warning_string,
                    ('No such platform: %s' if user_lang != 'zh' else '不支持设定的平台: %s') % sys.argv[3]),
                showSptPlatform(spt_type, spt_type_keys),
                qs_default_console.print(qs_error_string, 'Upload Failed' if user_lang != 'zh' else '上传失败!')
            )
        else:
            upload_image(path, plt_type=random.choice(spt_type_keys))


def ali_oss():
    """
    阿里云对象存储

    Ali Cloud object storage
    """
    try:
        op = sys.argv[2]
        if op not in ['-dl', '-up', '-ls', '-rm']:
            raise IndexError
        if op != '-ls' and '--bucket' in sys.argv:
            bucket = sys.argv[sys.argv.index('--bucket') + 1]
        elif op == '-ls':
            bucket = None if len(sys.argv) < 4 else sys.argv[3]
        else:
            bucket = None
        files = sys.argv[3:] if op != '-ls' else []
    except IndexError:
        qs_default_console.print(
            qs_info_string,
            'qs alioss:\n'
            '    -up <file> [bucket]: upload file to bucket\n'
            '    -dl <file> [bucket]: download file from bucket\n'
            '    -rm <file> [bucket]: remove file in bucket\n'
            '    -ls [bucket]       : get file info of bucket'
        ) if user_lang != 'zh' else qs_default_console.print(
            qs_info_string,
            'qs alioss:\n'
            '    -up <文件> [桶]: 上传文件至桶\n'
            '    -dl <文件> [桶]: 从桶下载文件\n'
            '    -rm <文件> [桶]: 从桶删除文件\n'
            '    -ls [桶]       : 获取桶文件信息'
        )
        return
    else:
        from .API.AliCloud import AliyunOSS
        ali_api = AliyunOSS()
        func_table = ali_api.get_func_table()
        for file in files:
            func_table[op](file, bucket)
        if not files:
            func_table[op](bucket)


def qiniu():
    """
    七牛云对象存储

    Qiniu cloud object storage
    """
    try:
        op = sys.argv[2]
        if op not in ['-dl', '-up', '-ls', '-rm']:
            raise IndexError
        if op != '-ls' and '--bucket' in sys.argv:
            bucket = sys.argv[sys.argv.index('--bucket') + 1]
        elif op == '-ls':
            bucket = None if len(sys.argv) < 4 else sys.argv[3]
        else:
            bucket = None
        files = sys.argv[3:] if op != '-ls' else []
    except IndexError:
        qs_default_console.print(
            qs_info_string,
            'qs qiniu:\n'
            '    -up <file> [bucket]: upload file to bucket\n'
            '    -dl <file> [bucket]: download file from bucket\n'
            '    -cp <url > [bucket]: copy file from url\n'
            '    -rm <file> [bucket]: remove file in bucket\n'
            '    -ls [bucket]       : get file info of bucket'
        ) if user_lang != 'zh' else qs_default_console.print(
            qs_info_string,
            'qs qiniu:\n'
            '    -up <文件> [桶]: 上传文件至桶\n'
            '    -dl <文件> [桶]: 从桶下载文件\n'
            '    -cp <链接> [桶]: 从链接下载文件到桶\n'
            '    -rm <文件> [桶]: 从桶删除文件\n'
            '    -ls [桶]       : 获取桶文件信息'
        )
        return
    else:
        from .API.QiniuOSS import QiniuOSS
        qiniu_api = QiniuOSS()
        func_table = qiniu_api.get_func_table()
        for file in files:
            func_table[op](file, bucket)
        if not files:
            func_table[op](bucket)


def txcos():
    """
    腾讯云对象存储

    Tencent Cloud object storage
    """
    try:
        op = sys.argv[2]
        if op not in ['-dl', '-up', '-ls', '-rm']:
            raise IndexError
        if op != '-ls' and '--bucket' in sys.argv:
            bucket = sys.argv[sys.argv.index('--bucket') + 1]
        elif op == '-ls':
            bucket = None if len(sys.argv) < 4 else sys.argv[3]
        else:
            bucket = None
        files = sys.argv[3:] if op != '-ls' else []
    except IndexError:
        qs_default_console.print(
            qs_info_string,
            'qs alioss:\n'
            '    -up <file> [bucket]: upload file to bucket\n'
            '    -dl <file> [bucket]: download file from bucket\n'
            '    -rm <file> [bucket]: remove file in bucket\n'
            '    -ls [bucket]       : get file info of bucket'
        ) if user_lang != 'zh' else qs_default_console.print(
            qs_info_string,
            'qs alioss:\n'
            '    -up <文件> [桶]: 上传文件至桶\n'
            '    -dl <文件> [桶]: 从桶下载文件\n'
            '    -rm <文件> [桶]: 从桶删除文件\n'
            '    -ls [桶]       : 获取桶文件信息'
        )
        return
    else:
        from .API.TencentCloud import TxCOS
        tx_api = TxCOS()
        func_table = tx_api.get_func_table()
        for file in files:
            func_table[op](file, bucket)
        if not files:
            func_table[op](bucket)


def translate():
    """
    qs默认的翻译引擎

    Qs default Translation engine
    """
    from . import trans_engine
    pyperclip = requirePackage('pyperclip')
    if trans_engine != 'default':
        from .API.TencentCloud import translate
    else:
        from .API.alapi import translate

    content = ' '.join(sys.argv[2:])
    if not content:
        try:
            content = pyperclip.paste()
        except:
            from . import qs_default_input
            content = qs_default_input.ask(
                'Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: '
                if user_lang != 'zh' else '抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:')
    if content:
        ret = translate(content.replace('\n', ' '))
        qs_default_console.print(ret) if ret else qs_default_console.log(qs_error_string, 'Translate Failed!')
    else:
        qs_default_console.log(
            qs_warning_string,
            "No content in your clipboard or command parameters!"
            if user_lang != 'zh' else
            '剪贴板或命令参数没有内容!')


def weather():
    """查天气 | Check weather"""
    from . import headers, dir_char
    from .ThreadTools import ThreadFunctionWrapper
    import requests

    def get_data(url):
        try:
            ct = requests.get(url, headers)
        except:
            return
        ct.encoding = 'utf-8'
        ct = ct.text.split('\n')
        if dir_char == '/':
            res = ct.copy()
        else:
            import re
            for line in range(len(ct)):
                ct[line] = re.sub('\x1b.*?m', '', ct[line])
            res = ct.copy()
        return res

    try:
        loc = sys.argv[2]
    except IndexError:
        loc = ''
    tls = [ThreadFunctionWrapper(get_data, 'https://wttr.in/' + (loc if loc else '?lang={}'.format(user_lang))),
           ThreadFunctionWrapper(get_data, 'https://v2.wttr.in/' + loc)]
    for i in tls:
        i.start()
    with qs_default_console.status('Requesting..' if user_lang != 'zh' else '请求中..') as st:
        for i in tls:
            i.join()
        st.update(status='Done')
    simple = tls[0].get_res()
    table = tls[1].get_res()
    if simple:
        if not loc:
            if user_lang == 'zh':
                from .API.alapi import translate
                enLocation = simple[0].split('：')[-1].strip()
                trans_loaction = translate(enLocation)
                qs_default_console.print('地区：' + trans_loaction if trans_loaction else enLocation)
            else:
                qs_default_console.print('Location' + simple[0][simple[0].index(':'):])
        simple = simple[2:7]
        print('\n'.join(simple))
    else:
        qs_default_console.log(qs_error_string, 'Get data failed.' if user_lang != 'zh' else '错误: 获取数据失败')
    if table:
        qs_default_console.print(table[3][:-1])
        bottom_line = 7
        try:
            while '╂' not in table[bottom_line]:
                bottom_line += 1
        except IndexError:
            qs_default_console.log(qs_error_string, 'Get Weather Data failed!' if user_lang != 'zh' else '获取天气数据失败')
            return
        for i in table[7:bottom_line + 2]:
            print(i[:-1])
        print('└────────────────────────────────────────────────────────────────────────')
        print('\n'.join(table[-3 if not loc else -4:]))
    else:
        print('Error: Get data failed.' if user_lang != 'zh' else '错误: 获取数据失败')


def largeImage():
    """
    百度图片效果增强

    Baidu picture effect enhancement
    """
    try:
        path = sys.argv[2]
    except IndexError:
        qs_default_console.log(qs_error_string, '%s: qs LG <img>' % 'Usage' if user_lang != 'zh' else '用法')
        return
    else:
        from .API.BaiduCloud import ImageDeal
        aip_cli = ImageDeal()
        with qs_default_console.status('Dealing..' if user_lang != 'zh' else '处理中..') as st:
            aip_cli.largeImage(path, st)
            st.update(status='Done')


def AipNLP():
    """百度NLP | Baidu NLP"""
    from .API.BaiduCloud import AipNLP
    pyperclip = requirePackage('pyperclip')
    ct = sys.argv[2:]
    if not ct:
        try:
            ct = [pyperclip.paste()]
        except:
            from . import qs_default_input
            ct = [qs_default_input.ask(
                'Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: '
                if user_lang != 'zh' else '抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:')]
    NLP = AipNLP()
    for _id, line in enumerate(ct):
        ct[_id] = NLP.get_res(line)
        if _id == 9:
            qs_default_console.print('...')
        elif _id < 9:
            qs_default_console.print(ct[_id])
    try:
        pyperclip.copy('\n'.join(ct))
    except:
        pass


def bili_cover():
    """下载Bilibili视频、直播的封面图片（视频链接、视频号均可识别）"""
    from .API.alapi import bili_cover as bc
    pyperclip = requirePackage('pyperclip')

    try:
        url = sys.argv[2]
    except IndexError:
        try:
            url = pyperclip.paste()
        except:
            qs_default_console.log(
                qs_error_string, 'Sorry, but your system may not be suppported by `pyperclip`'
                if user_lang != 'zh' else '抱歉，但是“pyperclip”不支持你的系统')
            return
    if not url:
        qs_default_console.log(
            qs_error_string, 'Usage: qs bcv <url | video code>'
            if user_lang != 'zh' else '用法: qs bcv <链接 | 视频码>')
        return
    bc(url)


def gbc():
    """查询中国垃圾分类（且仅支持中文查询）"""
    from .API.alapi import garbage_classification
    try:
        with qs_default_console.status('Requesting data..' if user_lang != 'zh' else '请求数据中..') as st:
            res = garbage_classification(sys.argv[2:])
            st.update(status='Done')
        qs_default_console.print(res, justify='center')
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
        qs_default_console.print(
            qs_error_string,
            'Usage: qs gbc <garbage...>' if user_lang != 'zh' else '用法: qs gbc <垃圾...>')


def short_video_info(son_call=False):
    """
    获取短视频信息 | Get short video information

    :return:
    """
    from .API.alapi import short_video_info
    from .NetTools import get_fileinfo, size_format
    pyperclip = requirePackage('pyperclip')
    import re
    try:
        url = sys.argv[2]
    except IndexError:
        try:
            url = pyperclip.paste()
            url = re.findall('https?://(?:[-\w.]|%[\da-fA-F]{2}|[/])+', url)[0]
        except:
            qs_default_console.print(
                qs_error_string, 'Sorry, but your system may not be suppported by `pyperclip`'
                if user_lang != 'zh' else '抱歉，但是“pyperclip”不支持你的系统')
            return
    if not url:
        qs_default_console.print(
            qs_error_string, 'Usage: qs svi <url/video code>' if not son_call else 'Usage: qs svd <url/video code>')
        return
    output_prefix = {
        'title': 'Title ' if user_lang != 'zh' else '标题',
        'video': 'Video ' if user_lang != 'zh' else '视频',
        'cover': 'Cover ' if user_lang != 'zh' else '封面',
        'source': 'Source' if user_lang != 'zh' else '来源'
    }
    with qs_default_console.status('正在获取视频信息') as _:
        status, res = short_video_info(url.strip('/'))
        if not status:
            qs_default_console.print(qs_info_string, res['title'] + ':' + res['source'])
            return status
        showStatus = '-url' in sys.argv
        qs_default_console.print(qs_info_string, '[{}] {}'.format(output_prefix['title'], res['title']))
        if res['video_url']:
            sz = int(get_fileinfo(res['video_url'])[-1].headers['content-length']) if not son_call else -1
            qs_default_console.print(
                qs_info_string,
                '[{}] {}\n{}'.format(
                    output_prefix['video'],
                    size_format(sz, True) if sz > 0 else '--',
                    res['video_url'] if showStatus else ''
                )
            )
        if showStatus:
            sz = int(get_fileinfo(res['cover_url'])[-1].headers['content-length'])
            qs_default_console.print(
                qs_info_string,
                '[{}] {}\n{}'.format(output_prefix['cover'], size_format(sz, True),
                                     res['cover_url'] if showStatus else '')
            )
    if system == 'darwin':
        from .ImageTools.ImagePreview import image_preview
        image_preview(res['cover_url'], True)
    if 'source' in res and res['source']:
        qs_default_console.print(qs_info_string, '[{}] {}'.format(output_prefix['source'], res['source']))
    return res


def short_video_dl():
    """
    下载短视频为mp4格式

    Download short video as mp4

    :return:
    """
    from .NetTools.NormalDL import normal_dl
    from .ImageTools.VideoTools import tomp4
    from . import remove

    res = short_video_info(son_call=True)

    if not res:
        qs_default_console.print(qs_error_string, 'Download failed' if user_lang != 'zh' else '下载失败')
        return

    fileName = res['title'] if res['title'] else 'UnknownTitle'
    if 'pics' in res and res['pics']:
        from .ImageTools.ImageTools import topng
        for index, url in enumerate(res['pics']):
            normal_dl(url, set_name=fileName + str(index + 1))
            topng(fileName + str(index + 1))
            remove(fileName + str(index + 1))
    elif res['video_url']:
        normal_dl(res['video_url'], set_name=fileName)
        tomp4(fileName)
        remove(fileName)


def acg():
    """
    获取随机acg图片链接（可选择下载）

    Get links to random ACG images (download optional)

    :return:
    """
    import random
    from .API.alapi import acg
    from .API.SimpleAPI import acg2

    st = qs_default_console.status('Requesting data..' if user_lang != 'zh' else '请求数据中..')
    st.start()
    try:
        status, acg_link, width, height = random.choice([acg, acg2])()
        qs_default_console.print(qs_info_string, f"{'链接' if user_lang == 'zh' else 'LINK'}: {acg_link}") \
            if status else qs_default_console.log(qs_error_string, acg_link)
        if status:
            qs_default_console.print(qs_info_string, '尺寸:' if user_lang == 'zh' else 'SIZE:', width, '×', height)
            if '--save' in sys.argv[2:]:
                from . import dir_char
                from .NetTools.NormalDL import normal_dl
                st.stop()
                acg_link = normal_dl(acg_link)  # disable download status for Windows
            from .ImageTools.ImagePreview import image_preview
            st.update(status='Loading image...\n' if user_lang != 'zh' else '加载图片中..\n')
            image_preview(open(acg_link) if '--save' in sys.argv[2:] else acg_link,
                          '--save' not in sys.argv[2:], qs_console_status=st)
            return
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
    finally:
        st.stop()


def bingImg():
    """
    获取bing图片链接（可选择下载）

    Get links to bing images (download optional)

    :return:
    """
    from .API.alapi import bingImg

    st = qs_default_console.status('Requesting data..' if user_lang != 'zh' else '请求数据中..')
    st.start()

    try:
        status, acg_link, cprt = bingImg()
        qs_default_console.print(qs_info_string, f"{'链接' if user_lang == 'zh' else 'LINK'}: {acg_link}") \
            if status else qs_default_console.log(qs_error_string, acg_link)
        if status:
            qs_default_console.print(qs_info_string, '版权:' if user_lang == 'zh' else 'CPRT:', cprt)
            if '--save' in sys.argv[2:]:
                from .NetTools.NormalDL import normal_dl
                st.stop()
                acg_link = normal_dl(acg_link)
            from .ImageTools.ImagePreview import image_preview
            st.update(status='Loading image...' if user_lang != 'zh' else '加载图片中..')
            image_preview(open(acg_link) if '--save' in sys.argv[2:] else acg_link,
                          '--save' not in sys.argv[2:], qs_console_status=st)
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
    finally:
        st.stop()


def preview_html_images():
    """
    获取网页中图片链接（可在Mac::iTerm中自动预览）

    Get links to pictures in the web page (automatically previewed in Mac::iTerm)

    :return:
    """
    from .API.SimpleAPI import imgs_in_url
    save_flag = '--save' in sys.argv
    if save_flag:
        sys.argv.remove('--save')
    for url in sys.argv[2:]:
        imgs_in_url(url, save_flag)


def kdCheck():
    """
    查国内快递

    Check domestic express

    :return:
    """
    from .API.alapi import kdCheck

    with qs_default_console.status('Requesting data..' if user_lang != 'zh' else '请求数据中..'):
        status, code, msg = kdCheck(sys.argv[2])
        if not status:
            qs_default_console.print(qs_error_string, msg)
            return

    from .TuiTools.Table import qs_default_table
    from . import table_cell
    from rich.text import Text

    width = qs_default_console.width // 4 * 3
    tb = qs_default_table(
        [
            {'header': "Time" if user_lang != 'zh' else '时间', 'justify': "center", 'style': "bold cyan"},
            {'header': "Description" if user_lang != 'zh' else '描述', 'justify': "center", 'no_wrap': False},
            {'header': "Status" if user_lang != 'zh' else '状态', 'justify': "center"},
        ], ([
                '[bold underline red]Unknown:heavy_exclamation_mark:', '[bold underline yellow]In transit:airplane:',
                '[bold underline green]In delivery:delivery_truck:', '[bold underline bold green]Signed receipt:hearts:'
            ][code] if user_lang != 'zh' else [
            '[bold underline red]未知:heavy_exclamation_mark:', '[bold underline yellow]运输中:airplane:',
            '[bold underline green]派送中:delivery_truck:', '[bold underline magenta]已签收:hearts:'
        ][code]) + '\n'
    )
    for info in msg[:-1] if code != 3 else msg:
        tb.add_row(
            info['time'], Text(table_cell(info['content'], width), justify='full')
            , '[green]:heavy_check_mark:'
        )
    if code != 3:
        tb.add_row(
            msg[-1]['time'], Text(table_cell(msg[-1]['content'], width), justify='full')
            , '[bold yellow]:arrow_left:'
        )
    qs_default_console.print(tb, justify="center")


def loli():
    """
    获取一张"可爱"萝莉图的URL，Mac+iTerm2下可在终端预览

    Get the URL of a "cute" Lori map, which can be previewed on the terminal under MAC + iterm2

    :return:
    """
    from .API.Lolicon import loli_img
    from .ImageTools import ImagePreview
    from .NetTools import NormalDL

    with qs_default_console.status('Requesting data..' if user_lang != 'zh' else '请求数据中..'):
        status, msg, data = loli_img()
        if not status:
            qs_default_console.print(qs_error_string, msg)
            return

    save_flag = '--save' in sys.argv
    proxy = ''

    if '-p' in sys.argv:
        from . import qs_config
        proxy = qs_config.basicSelect('default_proxy')

    for img in data:
        qs_default_console.print(f'[bold underline]{img["title"]} [dim]{img["author"]}', justify="center")
        qs_default_console.print(qs_info_string, '[bold]link' if user_lang != 'zh' else '[bold]链接', img['url'])
        qs_default_console.print(qs_info_string, '[bold]size' if user_lang != 'zh' else '[bold]尺寸',
                                 img['width'], 'x', img['height'])
        if save_flag:
            img['url'] = NormalDL.normal_dl(img['url'], set_proxy=proxy, set_referer='https://i.pximg.net')
        if system == 'darwin':
            ImagePreview.image_preview(open(img['url']) if save_flag else img['url'], not save_flag
                                       , set_proxy=proxy, set_referer='https://i.pximg.net')

        qs_default_console.print('-' * (qs_default_console.width // 4 * 3), justify='center')


def pinyin():
    from .API.alapi import pinyin

    content = ' '.join(sys.argv[2:])
    if not content:
        try:
            pyperclip = requirePackage('pyperclip')
            content = pyperclip.paste()
        except:
            from . import qs_default_input
            content = qs_default_input.ask(
                'Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: '
                if user_lang != 'zh' else '抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:')
    with qs_default_console.status('Requesting data..' if user_lang != 'zh' else '请求数据中..') as st:
        status, res = pinyin(content)
        st.update(status='Done' if user_lang != 'zh' else '请求完成')
    qs_default_console.print(qs_info_string, content)
    qs_default_console.print(qs_info_string if status else qs_error_string, res)


def setu():
    import random
    random.choice([acg, loli])()


def exchange():
    from .API.alapi import exchange

    st = qs_default_console.status('Requesting data..' if user_lang != 'zh' else '请求数据中..')
    st.start()
    try:
        status, data = exchange(sys.argv[3], float(sys.argv[2]))
        qs_default_console.print(f"{sys.argv[2]} {sys.argv[3]} ==> {data['exchange']} × {sys.argv[2]} = "
                                 f"{data['exchange_round']} {data['currency_to']}\n"
                                 f"{'Update' if user_lang != 'zh' else '更新时间'}: {data['update_time']}",
                                 justify="center") \
            if status else qs_default_console.log(qs_error_string, data)
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
    finally:
        st.stop()


def zhihuDaily():
    from .API.alapi import zhihuDaily
    from rich.panel import Panel
    from .ImageTools.ImagePreview import image_preview

    st = qs_default_console.status('Requesting data..' if user_lang != 'zh' else '请求数据中..')
    st.start()
    try:
        status, data = zhihuDaily()
        st.stop()
        if not status:
            qs_default_console.log(qs_error_string, data)
            return
        data = data['stories'] + data['top_stories']
        for item in data:
            res = '[bold cyan]' + ('Author: ' if user_lang != 'zh' else '作者: ')
            res += '[bold white]' + item['hint'] + '[/bold white]\n'
            res += '[bold cyan]' + ('Link:' if user_lang != 'zh' else '链接: ')
            res += '[bold blue]' + item['url'] + '[/bold blue]'
            if 'images' in item:
                if system == 'darwin':
                    image_preview(item['images'][0], True)
                else:
                    res += '\n[bold cyan]' + ('Image: \n' if user_lang != 'zh' else '图像: \n')
                    res += '[bold blue]  ' + '\n  '.join(item['images'])
            elif 'image' in item:
                image_preview(item['image'], True)
            qs_default_console.print(Panel(res, title='[b]' + item['title'], width=qs_default_console.width),
                                     justify="center", end='/n/n')
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
    finally:
        st.stop()


def wallhaven():
    from .API.SimpleAPI import wallhaven

    url, oneFlag = '', False
    if '-h' in sys.argv:
        return qs_default_console.print(qs_info_string, "Usage: qs wallhaven [--url <url>] [-one] [--save]")

    if '--url' in sys.argv:
        url = sys.argv[sys.argv.index('--url') + 1]
        sys.argv.remove('--url')
        sys.argv.remove(url)

    oneFlag = '-one' in sys.argv

    res = wallhaven(randomOne=oneFlag) if not url else wallhaven(url, randomOne=oneFlag)
    if not res:
        return

    if '--save' in sys.argv:
        from .NetTools.NormalDL import normal_dl
        import os
        for i in res:
            name = i['url'].split('/')[-1]
            if os.path.exists(name):
                qs_default_console.print(qs_warning_string,
                                         f'{name} is exists!' if user_lang != 'zh' else f'{name} 已存在!')
                continue
            qs_default_console.print(qs_info_string, f"Deal:\t{name}")
            qs_default_console.print(qs_info_string, f'Size:\t{" × ".join([str(j) for j in i["size"]])}')
            normal_dl(i['url'], output_error=True, failed2exit=True)
            qs_default_console.print('-' * qs_default_console.width)
    else:
        if oneFlag:
            res = res[0]
            qs_default_console.print(qs_info_string, 'URL:\t', res['url'])
            qs_default_console.print(qs_info_string, 'SIZE:\t', ' × '.join([str(i) for i in res['size']]))
            from .ImageTools.ImagePreview import image_preview

            with qs_default_console.status('Getting..' if user_lang != 'zh' else '获取图片中..') as st:
                image_preview(res['url'], True, qs_console_status=st)
        else:
            from .TuiTools.Table import qs_default_table

            tb = qs_default_table([
                {'header': "ID" if user_lang != 'zh' else '编号', 'justify': "center", 'style': "bold"},
                {'header': 'URL', 'justify': "center", 'style': "bold cyan"},
                {'header': 'Size' if user_lang != 'zh' else '尺寸', 'justify': "center"}
            ], 'Wallhaven Anime TopList\n')
            for iid, item in enumerate(res):
                tb.add_row(str(iid), item['url'], ' × '.join([str(i) for i in item['size']]))
            qs_default_console.print(tb, justify="center")


def lmgtfy():
    from .API.SimpleAPI import lmgtfy
    pyperclip = requirePackage('pyperclip')

    keyword = ' '.join(sys.argv[2:])
    res = lmgtfy(keyword)
    try:
        pyperclip.copy(res)
        qs_default_console.print(
            qs_info_string,
            'The generated link has been copied for you, and teach your friends how to use Google! (Can also Copy '
            'Following link'
            if user_lang != 'zh' else '已经为您复制生成的链接，快教朋友如何使用Google吧! (也可以复制下面的链接)'
        )
    except:
        qs_default_console.print(
            qs_warning_string,
            'Failed to use clipboard, you can copy the flowing link:'
            if user_lang != 'zh' else '调用剪切板失败，你可以复制下面的链接:'
        )
    qs_default_console.print(qs_info_string, res)


def daily60s():
    from .API.alapi import daily60s

    status, res = daily60s()
    if not status:
        return qs_default_console.print(qs_error_string, res)

    from rich.padding import Padding
    import signal
    import time

    cur_index = 0

    with qs_default_console.status('showing' if user_lang != 'zh' else '展示中') as st:
        def dealSignal(*argv):
            for index in range(cur_index + 1, len(res['news'])):
                qs_default_console.print(Padding(res['news'][index], (0, int(0.2 * qs_default_console.width))))
                qs_default_console.print()
            qs_default_console.print('-' * qs_default_console.width)
            qs_default_console.print(res['weiyu'] + '\n', justify='center')

            if '--save' in sys.argv:
                import os
                from .NetTools.NormalDL import normal_dl

                st.stop()
                name = normal_dl(res['image'], set_name='news.png')
                st.start()
                qs_default_console.print('Preview' if user_lang != 'zh' else '预览', justify='center')
                from .ImageTools.ImagePreview import image_preview

                st.update(status='loading image..' if user_lang != 'zh' else '加载图片中..')
                image_preview(open(name), qs_console_status=st)
            exit(0)

        signal.signal(signal.SIGINT, dealSignal)
        qs_default_console.print('[bold magenta underline]每天60秒读懂世界[/bold magenta underline]', justify='center')
        qs_default_console.print(res['date'], justify='center')
        qs_default_console.print('-' * qs_default_console.width)
        len_ls = [len(i) for i in res['news']]
        total_len = sum(len_ls)
        len_ls = [i / total_len * 60 for i in len_ls]
        len_ls[-1] = 0

        for index, item in enumerate(res['news']):
            qs_default_console.print(Padding(item, (0, int(0.2 * qs_default_console.width))))
            qs_default_console.print()
            cur_index = index
            time.sleep(len_ls[index])
    dealSignal()


def m2t():
    """
    磁力链接转种子文件

    Magnet link to Torrent file
    :return:
    """
    import re

    if '-f' in sys.argv:
        file_path = sys.argv[sys.argv.index('-f') + 1]
        with open(file_path, 'r') as f:
            contents = f.read()
    elif '-u' in sys.argv:
        contents = sys.argv[sys.argv.index('-u') + 1]
    else:
        contents = requirePackage('pyperclip', 'paste')()
    urls = re.findall('magnet:\?xt=urn:btih:(.*)', contents)
    if len(urls) > 1:
        from .__config__ import prompt
        url = prompt({
            'type': 'list',
            'name': 'hash',
            'message': 'Select hash code | 选择哈希码:',
            'choices': urls
        })['hash']
    else:
        url = urls[0]
    from .API.Lolicon import magnet2torrent
    magnet2torrent(url)


def d2m():
    """
    番号搜索并复制磁力链

    :return:
    """
    global url
    try:
        designation = sys.argv[2]
    except IndexError:
        return qs_default_console.print(qs_error_string, 'qs d2m <designation>')

    from .__config__ import prompt
    from .API.SimpleAPI import Designation2magnet

    copy = requirePackage('pyperclip', 'copy', not_ask=True)
    PyperclipException = requirePackage('pyperclip', 'PyperclipException')
    copied = False

    searcher = Designation2magnet(designation)
    infos = searcher.search_designation()
    choices = [f'[{n + 1}] ' + i[1] + ': ' + i[-1] for n, i in enumerate(infos)]

    try:
        url = searcher.get_magnet(
                infos[
                    choices.index(prompt({
                        'type': 'list',
                        'message': 'Select | 选择',
                        'name': 'sub-url',
                        'choices': choices
                    })['sub-url'])
                ][0]
        )
        if copy:
            copy(url)
            copied = True
        else:
            raise PyperclipException
    except KeyError:
        return
    except PyperclipException:
        qs_default_console.print(qs_info_string, url)

    if copied:
        qs_default_console.print(qs_info_string, 'magnet url copied to clipboard' if user_lang != 'zh' else '磁力链接已拷贝至粘贴板')


def doutu():
    """
    获取关键词的随机十张表情包
    :return:
    """
    try:
        keyword = sys.argv[2]
    except IndexError:
        return qs_default_console.print(qs_error_string, 'qs doutu <designation>')

    from .API.alapi import doutu
    from .ImageTools.ImagePreview import image_preview

    status, urls = doutu(keyword)
    if not status:
        qs_default_console.print(qs_error_string, urls)
    else:
        import random
        urls = random.choices(urls, k=10)

        for item in urls:
            qs_default_console.print(qs_info_string, item)
            image_preview(item, is_url=True)


def joke():
    """
    获取段子
    :return:
    """
    import re
    from .API.alapi import joke
    from rich.panel import Panel

    status, ct = joke()
    if not status:
        qs_default_console.print(qs_error_string, ct)
        return
    content = re.sub('&(.*?);', '', ct['content'])
    qs_default_console.print(Panel(content, title='[b magenta]' + ct['title'], title_align='center', width=qs_default_console.width, subtitle=ct['time']))



# coding=utf-8
"""
调用各种网络工具

Call various network tools
"""
import sys


def upgrade():
    """
    更新qs

    Upgrade qs
    """
    import os
    if os.system('pip3 install QuickStart-Rhy --upgrade'):
        os.system('pip install QuickStart-Rhy --upgrade')


def m3u8_dl(url):
    """
    下载m3u8

    Download *.m3u8
    """
    from .NetTools.M3u8DL import M3U8DL
    M3U8DL(url, url.split('.')[-2].split('/')[-1]).download()


def download():
    """
    qs下载引擎，使用--video or -v使用youtube-dl下载视频

    Qs download engine, use --video or -v use the default video download engine download
    """
    if any([i in sys.argv for i in ['-h', '-help', '--help']]):
        from . import user_lang, qs_default_console, qs_info_string
        qs_default_console.print(
            qs_info_string, 'Usage: "qs dl [url...]"\n'
            '  [--video] | [-v]  :-> download video (use youtube-dl)\n'
            '  [--proxy] | [-px] :-> use default proxy set in ~/.qsrc\n'
            '  [--name fileName] :-> Set File Name'
            if user_lang != 'zh' else
            '使用: "qs dl [链接...]"\n'
            '  [--video] | [-v]  :-> 使用youtube-dl下载视频\n'
            '  [--proxy] | [-px] :-> 使用配置表中的默认代理下载\n'
            '  [--name fileName] :-> 设置文件名')
        return
    global _real_main
    ytb_flag = '--video' in sys.argv or '-v' in sys.argv
    use_proxy = '--proxy' in sys.argv or '-px' in sys.argv
    other_args = []
    set_name = None
    if '--name' in sys.argv:
        set_name = sys.argv[sys.argv.index('--name') + 1]
    if ytb_flag or use_proxy or set_name:
        [sys.argv.remove(i) if i in sys.argv else None for i in ['--video', '-v', '--proxy', '-px', '--name', set_name]]
    nxt_flag = False
    for item in sys.argv[2:]:
        if item.startswith('-') or nxt_flag:
            if nxt_flag:
                nxt_flag = False
            if item.startswith('--'):
                nxt_flag = True
            other_args.append(item)

    [sys.argv.remove(item) for item in other_args]
    urls = sys.argv[2:]
    if not urls:
        from . import requirePackage
        pyperclip = requirePackage('pyperclip')
        urls = pyperclip.paste().split()
    if urls:
        if ytb_flag:
            from . import requirePackage
            from youtube_dl import _real_main
            _real_main = requirePackage('youtube_dl', '_real_main')
        from .NetTools.NormalDL import normal_dl
        from . import qs_config
        for url in urls:
            if url.endswith('.m3u8'):
                m3u8_dl(url)
            else:
                if use_proxy:
                    normal_dl(url, set_name=set_name, set_proxy=qs_config.basicSelect('default_proxy')) \
                        if not ytb_flag else _real_main([url, '--proxy', qs_config.basicSelect('default_proxy'),
                                                         '--merge-output-format', 'mp4'] + other_args)
                else:
                    normal_dl(url, set_name=set_name) \
                        if not ytb_flag else _real_main([url, '--merge-output-format', 'mp4'] + other_args)
    else:
        from . import user_lang, qs_default_console, qs_error_string
        qs_default_console.log(qs_error_string, "No url found!" if user_lang != 'zh' else '无链接输入')


def http():
    """
    开启http服务

    Turn on the http service.
    """
    url = ''
    if len(sys.argv) > 2:
        ip, port = sys.argv[2].split(':')
        port = int(port)
        if '-bind' in sys.argv:
            try:
                url = sys.argv[sys.argv.index('-bind') + 1]
                from .NetTools import formatUrl
                url = formatUrl(url)
            except IndexError:
                print('Usage: qs ftp ip:port -bind url')
                exit(0)
    else:
        from .NetTools import get_ip
        ip = get_ip()
        port = 8000
    if not ip:
        exit('get ip failed!')
    from .NetTools.HttpServer import HttpServers
    HttpServers(ip, port, url).start()


def netinfo():
    """
    通过域名或ip查询ip信息

    Query ip information via domain name or ip.
    """
    import socket
    import urllib.parse
    from . import user_lang, qs_default_console, qs_error_string, requirePackage
    pyperclip = requirePackage('pyperclip')
    from .API.alapi import ip_info

    def print_ip_info(info_ls):
        from .TuiTools.Table import qs_default_table

        table = qs_default_table([
            {'header': 'ip', 'justify': 'center', 'style': 'bold magenta'}, {'header': '运营商', 'justify': 'center'},
            {'header': '地址', 'justify': 'center'}, {'header': '经纬', 'justify': 'center'},
        ] if user_lang == 'zh' else [
            {'header': 'ip', 'justify': 'center', 'style': 'bold magenta'}, {'header': 'isp', 'justify': 'center'},
            {'header': 'pos', 'justify': 'center'}, {'header': 'location', 'justify': 'center'},
        ], 'NetInfo Results\n')
        for info in info_ls:
            table.add_row(info['ip'],
                          info['isp'].strip() if 'isp' in info else '未知', info['pos'],
                          str(info['location'])[1:-1].replace("'", ''))
        qs_default_console.print(table, justify="center")

    urls = sys.argv[2:] if len(sys.argv) > 2 else []
    if not urls:
        try:
            urls += pyperclip.paste().strip().split() if not urls else []
        except :
            from rich.prompt import Prompt
            urls = Prompt.ask(
                'Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: '
                if user_lang != 'zh' else '抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:'
            ).strip().split()
    if not urls:
        urls.append('me')
    res_ls = []
    for i in urls:
        try:
            addr = ''
            if i != 'me':
                if '://' in i:
                    protocol, domain = i[:i.index('://')], urllib.parse.urlparse(i).netloc
                    addr = socket.getaddrinfo(domain, protocol)[0][-1][0]
                else:  # 无网络协议或直接使用IP地址时默认采用http
                    addr = urllib.parse.urlparse(i).netloc
                    addr = socket.getaddrinfo(addr if addr else i, 'http')[0][-1][0]
            res_ls.append(ip_info(addr))
        except:
            qs_default_console.print(qs_error_string, 'Get domain failed:', i)
            continue
    print_ip_info(res_ls)


def wifi():
    """
    扫描附近wifi，选择连接

    Scan nearby wifi and select connection

    :return:
    """
    from . import system, user_lang, qs_default_console, qs_error_string, qs_info_string
    from .TuiTools.Table import qs_default_table
    if system.lower() != 'darwin':
        from .NetTools.WiFi import WiFi
    else:
        from .NetTools.WiFiDarwin import WiFi
    _wifi = WiFi()
    table = qs_default_table(
        ['id', 'ssid', 'signal', 'lock'] if user_lang != 'zh' else ['序号', '名称', '信号', '加密'],
        title='Wi-Fi')
    connectable_wifi = _wifi.scan()
    if not connectable_wifi:
        qs_default_console.print(qs_error_string, "No available wifi" if user_lang != 'zh' else '没有可用的wifi')
        return

    from .__config__ import prompt
    from prompt_toolkit.validation import Validator, ValidationError

    class ssidValidator(Validator):
        def validate(self, document):
            document = document.text
            try:
                if int(document) >= len(connectable_wifi):
                    raise IndexError
            except:
                raise ValidationError(message='请输入合法序号' if user_lang == 'zh' else 'Input validate id', cursor_position=len(document))
            return True

    questions = [{
        'type': 'input',
        'name': 'ssid',
        'message': '选择序号以连接:' if user_lang == 'zh' else 'Choose id to connect:',
        'validate': ssidValidator
    }, {
        'type': 'password',
        'name': 'password',
        'message': '输入密码:' if user_lang == 'zh' else 'Input password:'
    }]

    for i, l in enumerate(connectable_wifi):
        table.add_row(*([str(i)] + [str(j) for j in l]))
    qs_default_console.print(table, justify="center")

    res = prompt(questions)
    if not res:
        return
    ssid = connectable_wifi[int(res['ssid'])]
    password = res['password']
    qs_default_console.print(qs_info_string, 'Connect succeed' if user_lang != 'zh' else '连接成功') \
        if _wifi.conn(ssid, password) else \
        qs_default_console.print(qs_error_string, 'Connect failed' if user_lang != 'zh' else '连接失败')



"""
在终端预览图片 | 目前仅有MacOS下的iTerm可用, 你需要自行安装imgcat库

preview image on terminal | At present, only iTerm under MacOS is available,
you need to install imgcat library by yourself
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from .. import qs_default_console, qs_console_width
from .. import headers, prompt, user_lang, force_show_img
import math
import base64
import sys
import os
import struct
import io
import subprocess

# 记录强制显示状态，避免重复询问
force_show_option = False
has_set_force_show_option = False

TMUX_WRAP_ST = b'\033Ptmux;'
TMUX_WRAP_ED = b'\033\\'

OSC = b'\033]'
CSI = b'\033['
ST  = b'\a'      # \a = ^G (bell)


def get_image_shape(buf):
    '''
    Extracts image shape as 2-tuple (width, height) from the content buffer.

    Supports GIF, PNG and other image types (e.g. JPEG) if PIL/Pillow is installed.
    Returns (None, None) if it can't be identified.
    '''
    def _unpack(fmt, buffer, mode='Image'):
        try:
            return struct.unpack(fmt, buffer)
        except struct.error:
            raise ValueError("Invalid {} file".format(mode))

    # TODO: handle 'stream-like' data efficiently, not storing all the content into memory
    L = len(buf)

    if L >= 10 and buf[:6] in (b'GIF87a', b'GIF89a'):
        return _unpack("<hh", buf[6:10], mode='GIF')
    elif L >= 24 and buf.startswith(b'\211PNG\r\n\032\n') and buf[12:16] == b'IHDR':
        return _unpack(">LL", buf[16:24], mode='PNG')
    elif L >= 16 and buf.startswith(b'\211PNG\r\n\032\n'):
        return _unpack(">LL", buf[8:16], mode='PNG')
    else:
        # everything else: get width/height from PIL
        # TODO: it might be inefficient to write again the memory-loaded content to buffer...
        b = io.BytesIO()
        b.write(buf)

        try:
            from PIL import Image
            im = Image.open(b)
            return im.width, im.height
        except (IOError, OSError) as ex:
            # PIL.Image.open throws an error -- probably invalid byte input are given
            sys.stderr.write("Warning: PIL cannot identify image; this may not be an image file" + "\n")
        except ImportError:
            # PIL not available
            sys.stderr.write("Warning: cannot determine the image size; please install Pillow" + "\n")
            sys.stderr.flush()
        finally:
            b.close()

        return None, None


def _isinstance(obj, module, clsname):
    """A helper that works like isinstance(obj, module:clsname), but even when
    the module hasn't been imported or the type is not importable."""

    if module not in sys.modules:
        return False

    try:
        clstype = getattr(sys.modules[module], clsname)
        return isinstance(obj, clstype)
    except AttributeError:
        return False


def to_content_buf(data):
    # TODO: handle 'stream-like' data efficiently, rather than storing into RAM

    if isinstance(data, bytes):
        return data

    elif isinstance(data, io.BufferedReader):
        buf = data
        return buf.read()

    elif isinstance(data, io.TextIOWrapper):
        return data.buffer.read()

    elif _isinstance(data, 'numpy', 'ndarray'):
        # numpy ndarray: convert to png
        im = data
        if len(im.shape) == 2:
            mode = 'L'     # 8-bit pixels, grayscale
            im = im.astype(sys.modules['numpy'].uint8)
        elif len(im.shape) == 3 and im.shape[2] in (3, 4):
            mode = None    # RGB/RGBA
            if im.dtype.kind == 'f':
                im = (im * 255).astype('uint8')
        else:
            raise ValueError("Expected a 3D ndarray (RGB/RGBA image) or 2D (grayscale image), "
                             "but given shape: {}".format(im.shape))

        try:
            from PIL import Image
        except ImportError as e:
            raise ImportError(e.msg +
                              "\nTo draw numpy arrays, we require Pillow. " +
                              "(pip install Pillow)")       # TODO; reraise

        with io.BytesIO() as buf:
            # mode: https://pillow.readthedocs.io/en/4.2.x/handbook/concepts.html#concept-modes
            Image.fromarray(im, mode=mode).save(buf, format='png')
            return buf.getvalue()

    elif _isinstance(data, 'torch', 'Tensor'):
        # pytorch tensor: convert to png
        im = data
        try:
            from torchvision import transforms
        except ImportError as e:
            raise ImportError(e.msg +
                              "\nTo draw torch tensor, we require torchvision. " +
                              "(pip install torchvision)")

        with io.BytesIO() as buf:
            transforms.ToPILImage()(im).save(buf, format='png')
            return buf.getvalue()

    elif _isinstance(data, 'tensorflow.python.framework.ops', 'EagerTensor'):
        im = data
        return to_content_buf(im.numpy())

    elif _isinstance(data, 'PIL.Image', 'Image'):
        # PIL/Pillow images
        img = data

        with io.BytesIO() as buf:
            img.save(buf, format='png')
            return buf.getvalue()

    elif _isinstance(data, 'matplotlib.figure', 'Figure'):
        # matplotlib figures
        fig = data
        if fig.canvas is None:
            from matplotlib.backends.backend_agg import FigureCanvasAgg
            FigureCanvasAgg(fig)

        with io.BytesIO() as buf:
            fig.savefig(buf)
            return buf.getvalue()

    else:
        raise TypeError("Unsupported type : {}".format(type(data)))


def get_tty_size():
    with open('/dev/tty') as tty:
        rows, columns = subprocess.check_output(['stty', 'size'], stdin=tty).split()
    return int(rows), int(columns)


def real_height(buf, pixels_per_line=24):
    im_width, im_height = get_image_shape(buf)
    if im_height:
        assert pixels_per_line > 0
        height = (im_height + (pixels_per_line - 1)) // pixels_per_line

        # automatically limit height to the current tty,
        # otherwise the image will be just erased
        try:
            tty_height, _ = get_tty_size()
            return max(1, min(height, tty_height - 9))
        except OSError:
            # may not be a terminal
            pass
    else:
        # image height unavailable, fallback?
        return 10


def imgcat(buf, width=None, height=None, preserve_aspect_ratio=True, fp=None):
    """
    Print image on terminal (iTerm2).

    Follows the file-transfer protocol of iTerm2 described at
    https://www.iterm2.com/documentation-images.html.

    Args:
        :param buf: the content of image in buffer interface, numpy array, etc.
        :param width: the width for displaying image, in number of characters (columns)
        :param height: the height for displaying image, in number of lines (rows)
        :param fp: The buffer to write to, defaults sys.stdout
        :param filename:
    """
    if fp is None:
        fp = sys.stdout.buffer  # for stdout, use buffer interface (py3)

    if len(buf) == 0:
        raise ValueError("Empty buffer")

    if height is None:
        height = real_height(buf)

    # need to detect tmux
    is_tmux = 'TMUX' in os.environ and 'tmux' in os.environ['TMUX']

    # tmux: print some margin and the DCS escape sequence for passthrough
    # In tmux mode, we need to first determine the number of actual lines
    if is_tmux:
        fp.write(b'\n' * height)
        # move the cursers back
        fp.write(CSI + b'?25l')
        fp.write(CSI + str(height).encode() + b"F")     # PEP-461
        fp.write(TMUX_WRAP_ST + b'\033')

    # now starts the iTerm2 file transfer protocol.
    fp.write(OSC)
    fp.write(b'1337;File=inline=1')
    fp.write(b';size=' + str(len(buf)).encode())
    fp.write(b';height=' + str(height).encode())
    if width:
        fp.write(b';width=' + str(width).encode())
    if not preserve_aspect_ratio:
        fp.write(b';preserveAspectRatio=0')
    fp.write(b':')
    fp.flush()

    buf_base64 = base64.b64encode(buf)
    fp.write(buf_base64)

    fp.write(ST)

    if is_tmux:
        # terminate DCS passthrough mode
        fp.write(TMUX_WRAP_ED)
        # move back the cursor lines down
        fp.write(CSI + str(height).encode() + b"E")
        fp.write(CSI + b'?25h')
    else:
        fp.write(b'\n')

    # flush is needed so that the cursor control sequence can take effect
    fp.flush()


def image_preview(img, is_url=False, set_proxy: str = '', set_referer: str = '',
                  qs_console_status=None, set_width_in_rc_file: int = 0, force_show: bool = force_show_img):
    """
    在终端预览图片 | 目前仅有MacOS下的iTerm可用, 但你可以开启强制显示选项预览

    preview image on terminal | At present, only iTerm under MacOS is available, but you can enable force show option to preview

    :param force_show: 强制显示图片，即使终端非iTerm2
    :param set_width_in_rc_file: 设置图片宽度，以像素为单位，默认为0，即不设置
    :param is_url: 是否为url
    :param img: opened file, numpy array, PIL.Image, matplotlib fig
    :param set_proxy: set proxy
    :param set_referer: set refer
    :param qs_console_status: qs终端的状态信息
    :return:
    """
    global force_show_option, has_set_force_show_option
    try:
        if not (force_show_option or force_show or (not has_set_force_show_option and prompt({
            'type': 'confirm',
            'name': 'preview_image_on_terminal',
            'message': 'Preview image on this terminal?' if user_lang != 'zh' else '确认在该终端预览图片?',
            'default': False,
        })['preview_image_on_terminal'])):
            has_set_force_show_option = True
            return
        else:
            has_set_force_show_option = True
            force_show_option = True
        if not is_url and (isinstance(img, str) and not os.path.exists(img)):
            is_url = img.startswith('http')
        if is_url:
            # from PIL import Image
            from .. import requirePackage
            from io import BytesIO
            import requests
            Image = requirePackage('PIL', 'Image', 'Pillow')
            if set_referer:
                headers['referer'] = set_referer
            proxies = {
                'http': 'http://' + set_proxy,
                'https': 'https://' + set_proxy
            } if set_proxy else {}
            res = requests.get(img, headers=headers, proxies=proxies).content
            if qs_console_status:
                qs_console_status.update(status='Opening')
            img = Image.open(BytesIO(res))
        elif not is_url and isinstance(img, str) and os.path.exists(img):
            from .. import requirePackage
            Image = requirePackage('PIL', 'Image', 'Pillow')
            img = Image.open(img)
        if qs_console_status:
            qs_console_status.stop()

        buf = to_content_buf(img)
        width, height = get_image_shape(buf)
        _real_height = real_height(buf)
        _real_width = math.ceil(width / height * _real_height) * 2 + 1

        console_width = qs_console_width if not set_width_in_rc_file else set_width_in_rc_file
        qs_default_console.print(' ' * int(max((console_width - _real_width) / 2 - 1, 0)), end='')

        imgcat(buf, height=real_height(buf))
    except Exception as e:
        if qs_console_status:
            qs_console_status.stop()
        from .. import qs_error_string
        qs_default_console.print(qs_error_string, repr(e))
        return



"""
Generate Various Table
"""


def qs_default_table(headers: list, title: str = ''):
    from rich.table import Table
    from rich.box import SIMPLE_HEAVY

    res_table = Table(show_edge=False, row_styles=['none', 'dim'], box=SIMPLE_HEAVY, title=('[bold underline]' + title if title else ''))
    for i in headers:
        if isinstance(i, str):
            res_table.add_column(i, justify='center')
        elif isinstance(i, dict):
            res_table.add_column(**i)
        else:
            res_table.add_column(*i)

    return res_table



# coding=utf-8
"""
线程工具

Threading tools
"""
import threading
from concurrent.futures import ThreadPoolExecutor, wait


class ThreadFunctionWrapper(threading.Thread):
    def __init__(self, function, *args, **kwargs):
        """
        开辟子线程执行任务（初始化）

        Create child threads to perform tasks (initialization)

        :param function: 函数
        :param args: 参数
        :param kwargs: 参数
        """
        threading.Thread.__init__(self)
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.res = None

    def run(self):
        """
        执行任务，结果保存在self.res

        Execute the task and save the results in self.res

        :return: None
        """
        self.res = self.function(*self.args, **self.kwargs)

    def get_res(self):
        """
        获取任务执行结果

        Gets the task execution results

        :return: self.res
        """
        return self.res


class FileWriters:
    def __init__(self, filename: str, workers: int, mode: str):
        """
        写文件线程池，为了适配密集io而设计的并行写算法。(采用多个文件指针加速)

        Write file thread pool, a parallel write algorithm designed to accommodate dense IO.
        (Multiple file pointer acceleration)

        :param filename: 文件名
        :param workers: 线程数
        :param mode: 读写模式，如'wb', 'rb+'
        """
        self.workers = workers
        self.handles = [open(filename, mode) for i in range(workers)]
        self.handles_lock = [threading.Lock() for i in range(workers)]
        self.pool = ThreadPoolExecutor(max_workers=workers)
        self.job_q = []
        self.cur_handle = 0

    def wait(self):
        """
        等待完全完成任务

        Wait for the task to complete

        :return:
        """
        wait(self.job_q)

    def _write(self, handle: int, content, index):
        fp = self.handles[handle]
        self.handles_lock[handle].acquire()
        fp.seek(index)
        fp.write(content)
        self.handles_lock[handle].release()

    def new_job(self, content: bytes, index: int):
        """
        写入新的文件块

        Write a new file block

        :param content: 文件内容
        :param index: 起始位置
        :return:
        """
        self.job_q.append(self.pool.submit(self._write, self.cur_handle, content, index))
        self.cur_handle = (self.cur_handle+1) % self.workers

    def __del__(self):
        """
        删除对象，必须等待线程池工作结束

        To delete an object, you must wait for the thread pool to finish working

        :return:
        """
        self.wait()
        self.pool.shutdown()



from . import size_format, get_fileinfo
from concurrent.futures import ThreadPoolExecutor, wait
from ..SystemTools import get_core_num
from ..TuiTools.Bar import NormalProgressBar
from .. import user_lang, qs_default_console, qs_error_string, qs_info_string, qs_warning_string, headers, dir_char
from threading import Lock
from requests import get
import os

core_num = min(get_core_num() * 4, 16)


class MultiSingleDL:
    def __init__(self, urls: list, rt_dir: str, proxy: str = '', referer: str = '', failed2exit: bool = False):
        self.proxy = proxy
        self.referer = referer
        self.proxies = {
            'http': 'http://' + proxy,
            'https': 'https://' + proxy
        } if proxy else {}
        self.headers = headers
        if referer:
            self.headers['Referer'] = referer
        self.rt_dir = rt_dir if rt_dir.endswith(dir_char) else rt_dir + dir_char
        self.failed2exit = failed2exit
        self.status_dict = {}
        self.infos = {}
        self.urls = set(urls)
        self.task_num = len(self.urls)
        self.cur_task_num = 0
        self.task_num_lock = Lock()
        self.pool = ThreadPoolExecutor(max_workers=core_num)
        self.status = qs_default_console.status('获取文件信息中...')
        self.progress, self.task_id = NormalProgressBar('多文件下载', self.task_num)

    def _info(self, url):
        real_url, name, r = get_fileinfo(url, self.proxy, self.referer)

        info_flag = True
        if not (url and name and r):
            info_flag = False
            qs_default_console.print(qs_error_string if self.failed2exit else qs_warning_string, url, ':',
                                     'Get File information failed, please check network!'
                                     if user_lang != 'zh' else '获取文件信息失败，请检查网络!')
            if self.failed2exit:
                self.enabled = False
                return

        self.infos[url] = {
            'url': real_url,
            'name': os.path.basename(url) if name and '.' not in name else name,
            'size': -1 if not info_flag else int(r.headers['content-length']) if 'content-length' in r.headers else -1
        }

        self.task_num_lock.acquire()
        self.cur_task_num += 1
        self.status.update(f'获取文件信息中... {self.cur_task_num} / {self.task_num}')
        self.task_num_lock.release()

    def _dl(self, url, filename):
        r = get(self.infos[url]['url'], stream=True, proxies=self.proxies, headers=self.headers)
        with open(self.rt_dir + filename, 'wb') as f:
            for chunk in r.iter_content(32768):
                f.write(chunk)
        self.progress.advance(self.task_id, 1)

    def run(self, name_map: dict = None):
        self.status.start()
        wait([self.pool.submit(self._info, item) for item in self.urls])

        total = sum([self.infos[i]['size'] for i in self.infos])
        failed_num = [self.infos[i]['size'] for i in self.infos].count(-1)
        qs_default_console.print(qs_info_string, f'获取文件信息完毕! 文件总大小: {size_format(total)} | 未获取到详细信息条数为: {failed_num}')
        self.status.stop()

        self.progress.start()
        wait([self.pool.submit(self._dl, item, self.infos[item]['name']) for item in self.urls] if not name_map else [self.pool.submit(self._dl, item, name_map[item]) for item in self.urls])
        self.progress.stop()

        return [self.rt_dir + self.infos[i]['name'] for i in self.infos] if not name_map else [self.rt_dir + name_map[i] for i in self.infos]


def multi_single_dl(urls: list, rt_dir: str = './', proxy: str = '', referer: str = '',
                    failed2exit: bool = False, name_map: dict = None):
    """
    并行多个小文件下载

    :param urls: 小文件的URL
    :param rt_dir: 文件下载目录
    :param proxy: 代理
    :param referer: referer
    :param failed2exit: 信息获取失败立即退出
    :param name_map: 文件命名映射{URL: filename}
    :return: 下载好的文件路径列表
    """
    return MultiSingleDL(urls=urls, rt_dir=rt_dir, proxy=proxy, referer=referer, failed2exit=failed2exit).run(
        name_map=name_map
    )



# coding=utf-8
"""
Docs for different language:
    中文: https://rhythmlian.cn/2020/08/09/QuickStart-Rhy-zh/

    English: https://rhythmlian.cn/2020/02/14/QuickStart-Rhy/
"""
import os
import sys
from .__config__ import QsConfig, QsCache, dir_char, system, qs_default_console, prompt
from .NetTools import headers
from rich.prompt import Prompt as qs_default_input

name = 'QuickStart_Rhy'

user_root = os.path.expanduser('~') + dir_char
qs_config = QsConfig(user_root + '.qsrc', os.path.exists(user_root + '.qsrc'))
qs_cache = QsCache(user_root + '.qs_cache')

user_lang = qs_config.basicSelect('default_language')
user_currency = qs_config.basicSelect('default_currency')
trans_engine = qs_config.basicSelect('default_translate_engine')['support']\
[qs_config.basicSelect('default_translate_engine')['index']]
user_pip = qs_config.basicSelect('default_pip')
force_show_img = qs_config.basicSelect('force_show_img')

qs_error_string = f'[bold red][{"ERROR" if user_lang != "zh" else "错误"}]'
qs_warning_string = f'[bold yellow][{"WARNING" if user_lang != "zh" else "警告"}]'
qs_info_string = f'[bold cyan][{"INFO" if user_lang != "zh" else "提示"}]'
qs_console_width = qs_default_console.width


def requirePackage(pname: str, module: str = "", real_name: str = "", not_exit: bool = True, not_ask: bool = False, set_pip: str = user_pip):
    """
    获取本机上的python第三方库，如没有则询问安装

    :param not_ask: 不询问，无依赖项则报错
    :param set_pip: 设置pip路径
    :param pname: 库名
    :param module: 待引入的模块名，可缺省
    :param real_name: 用于 pip3 install 的名字
    :param not_exit: 安装后不退出
    :return: 库或模块的地址
    """
    try:
        exec(f'from {pname} import {module}' if module else f"import {pname}")
    except (ModuleNotFoundError, ImportError):
        if not_ask:
            return None
        confirm = prompt({
            'type': 'confirm',
            'name': 'install',
            'message': f"""Qs require {pname + (' -> ' + module if module else '')}, confirm to install?  
  Qs 依赖 {pname + (' -> ' + module if module else '')}, 是否确认安装?""",
            'default': True})['install']
        if confirm:
            os.system(f'{set_pip} install {pname if not real_name else real_name} -U')
            if not_exit:
                exec(f'from {pname} import {module}' if module else f"import {pname}")
            else:
                qs_default_console.print(qs_info_string, f'just run again: "{" ".join(sys.argv)}"')
                exit(0)
        else:
            exit(-1)
    finally:
        return eval(f'{module if module else pname}')


def cut_string(string: str, length: int) -> list:
    """
    每隔l个字符切分字符串

    :param string: 字符串
    :param length: 切分长度
    :return: 切分后产生的list
    """
    string = string.strip().replace('\n', ' ')
    res, cur, cnt = [], '', 0
    for i in string:
        cnt += 2 if ord(i) > 255 else 1
        if cnt <= length:
            cur += i
        else:
            res.append(cur)
            cur, cnt = i, 2 if ord(i) > 255 else 1
    if cur:
        res.append(cur)
    return res


def table_cell(string: str, length: int) -> str:
    return ' '.join(cut_string(string, length))


def deal_ctrl_c(signum, frame):
    """
    默认的处理ctrl c的函数

    default function to deal [CTRL C]

    :param signum: signum
    :param frame: frame
    :return: None
    """
    if signum or frame or True:
        exit(0)


def remove(path):
    """
    删除文件或文件夹

    delete file or folder.

    :param path: 路径
    :return: None
    """
    if os.path.exists(path):
        if os.path.isdir(path):
            import shutil
            shutil.rmtree(path)
        else:
            os.remove(path)


def cur_time():
    """
    获取当前时间

    get time of now

    :return: None
    """
    week = {
        'Monday': '周一',
        'Tuesday': '周二',
        'Wednesday': '周三',
        'Thursday': '周四',
        'Friday': '周五',
        'Saturday': '周六',
        'Sunday': '周日'
    }
    import time
    tm = time.strftime('%Y年%m月%d日 %A %H:%M:%S', time.localtime(time.time())).split()
    tm[1] = week[tm[1]]
    qs_default_console.print(qs_info_string, ' '.join(tm))


def open_url(argv: list = None):
    """
    打开命令行参数中的链接

    open urls in argv

    :return: None
    """
    import webbrowser as wb
    from .NetTools import formatUrl
    if not argv:
        argv = sys.argv[2:]
    if argv:
        for url in argv:
            url = formatUrl(url)
            wb.open_new_tab(url)
    else:
        pyperclip = requirePackage('pyperclip')
        try:
            url = pyperclip.paste()
        except :
            url = qs_default_input.ask(
                'Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: '
                if user_lang != 'zh' else '抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:'
                , console=qs_default_console)
        wb.open_new_tab(formatUrl(url))


def open_app():
    """
    Mac OS下的open（不支持其他平台）

    open app (only support Mac OS)

    :return: None
    """
    if system == 'darwin':
        os.system('open -a ' + ' '.join([i.replace(' ', '\\ ') for i in sys.argv[2:]]))
    else:
        qs_default_console.print(qs_error_string, '"-a" is only support Mac OS X')


def open_file(argv=None):
    """
    使用合适应用打开文件

    Open file with appropriate application

    :return: None
    """
    if not argv:
        argv = [i.replace(' ', '\\ ') for i in sys.argv[2:]]
    if system == 'darwin':
        os.system('open ' + ' '.join(argv))
    elif system == 'linux':
        from subprocess import run
        run(['xdg-open'] + [i for i in argv])
    else:
        import webbrowser as wb

        for file in argv:
            if os.path.exists(file):
                path = os.path.abspath(file)
                wb.open('file://%s' % path)


def init():
    import webbrowser as wb
    wb.open('http://login.cup.edu.cn')


def calculate():
    """
    执行算数表达式

    Execute the arithmetic expression.

    :return: None
    """
    try:
        exp = ' '.join(sys.argv[2:])
        qs_default_console.print('%s = %s' % (exp, eval(exp)))
    except Exception as e:
        qs_default_console.print(qs_info_string, 'Usage: qs cal <exp like "1+1">')
        qs_default_console.print(qs_error_string, repr(e))


def pcat():
    """
    输出粘贴板中内容

    Output the contents of the clipboard
    :return: None
    """
    print(requirePackage('pyperclip', 'paste')())


def fcopy():
    """
    获取文件内容并复制进粘贴板

    Get the content of the file and copy it into the pasteboard
    :return:
    """
    if not os.path.exists(sys.argv[2]):
        return qs_default_console.print(qs_error_string, "No such file:" if user_lang != 'zh' else '未找到文件:', sys.argv[2])
    with open(sys.argv[2], 'r') as f:
        requirePackage('pyperclip').copy(f.read())


def get_user_lang():
    print(user_lang)


def play_music():
    AS = requirePackage('pydub', 'AudioSegment')
    play = requirePackage('pydub.playback', 'play')

    for music in sys.argv[2:]:
        try:
            play(AS.from_file(music))
        except:
            pass



# coding=utf-8
"""
命令交互，查看qs的各种菜单和文档

Command interaction, view various menus and documents of QS
"""
from . import user_lang, open_url
from .__config__ import prompt


def onlineDocuments():
    res = prompt({
        'type': 'list',
        'name': 'onlineSource',
        'message': 'Switch a website source:' if user_lang != 'zh' else '选择合适你的网站源:',
        'choices': [
            '1. CDN in China' if user_lang != 'zh' else '1. 国内CDN',
            '2. Github Page (non-Chinese users)' if user_lang != 'zh' else '2. Github Page (非中国用户)',
        ]
    })['onlineSource']
    import webbrowser as wb
    wb.open(
        {
            '1': 'https://rhythmlian.cn/' + ('2020/02/14/QuickStart-Rhy/'
                                             if user_lang != 'zh' else '2020/08/09/QuickStart-Rhy-zh/'),
            '2': 'https://rhythmicc.github.io/' + ('2020/02/14/QuickStart-Rhy/'
                                                   if user_lang != 'zh' else '2020/08/09/QuickStart-Rhy-zh/'),
        }[res[0]]
    )


def lesson():
    mainMenu = {
        'type': 'list',
        'name': 'action',
        'message': 'Welcome using qs, choose and get help:' if user_lang != 'zh' else '欢迎使用qs, 选择以下选项获得帮助:',
        'choices': [
            '1. basic tools' if user_lang != 'zh' else '1. 基础工具',
            '2. system tools' if user_lang != 'zh' else '2. 系统工具',
            '3. network tools' if user_lang != 'zh' else '3. 网路工具',
            '4. api tools' if user_lang != 'zh' else '4. 扩展工具',
            '5. image tools' if user_lang != 'zh' else '5. 图像工具',
            '6. online Docs' if user_lang != 'zh' else '6. 在线文档',
            '7. join TG group' if user_lang != 'zh' else '7. 加入TG群'
        ]
    }
    res = prompt(mainMenu)['action']

    if res[0] == '7':
        return open_url(['https://t.me/joinchat/G2mpk7-S85eM7sb7'])
    else:
        from .funcList import menu_table
        {
            '1': menu_table['basic'],
            '2': menu_table['system'],
            '3': menu_table['net'],
            '4': menu_table['api'],
            '5': menu_table['image'],
            '6': onlineDocuments,
        }[res[0]]()



# coding=utf-8
from . import *
from .. import user_currency
import os
import json
from urllib.parse import quote
import requests

__common_token__ = 'hUxPTdnybk1XLUEFtzkj'
alapi_token = pre_check('alapi_token', False)
if not alapi_token:
    from .. import qs_default_console, qs_error_string, qs_warning_string
    from ..__config__ import prompt

    qs_default_console.print(qs_error_string, 'This function require alapi token!\n这个功能需要alapi token\n')
    if prompt({
        'type': 'confirm',
        'name': 'confirm',
        'message': 'To get alapi token?\n是否前往获取alapi token?',
        'default': True
    })['confirm']:
        from .. import open_url
        from .. import qs_config

        try:
            open_url(['https://www.alapi.cn/'])
        except Exception as e:
            qs_default_console.print(qs_error_string, 'Failed to open browser, try url: https://www.alapi.cn/')
        finally:
            alapi_token = prompt({
                'type': 'input',
                'name': 'token',
                'message': 'Input alapi token | 输入alapi token:',
            })['token']
            qs_config.apiUpdate('alapi_token', alapi_token)
    elif prompt({
        'type': 'confirm',
        'name': 'confirm',
        'message': 'Use common but limited token?\n是否愿意使用公共但是受限的token?',
        'default': True
    })['confirm']:
        alapi_token = __common_token__
        qs_config.apiUpdate('alapi_token', alapi_token)
        qs_default_console.print(
            qs_warning_string,
            'Tokens with restricted applications may not be able to use the functions provided by alapi normally, '
            'and you can try again multiple times.'
            if user_lang != 'zh' else '应用受限的token可能无法正常使用alapi提供的功能，可多次重新尝试。')
    else:
        exit()

if not pre_check('__ban_warning', False) and alapi_token == __common_token__:
    from .. import qs_default_console, qs_warning_string
    qs_default_console.print(
        qs_warning_string, 'Using common but limited token.' if user_lang != 'zh' else '正在使用公共但受限的token')
    qs_default_console.print(
        qs_warning_string,
        'If you do not want this, try to apply one:' if user_lang != 'zh' else '如果您不想这样, 可以来申请一个自己的:',
        'https://www.alapi.cn/')

v2_url = "https://v2.alapi.cn/api/"


def _upimg_get_avaliable_platform():
    json_string = requests.get(v2_url + 'image/type', headers={'token': alapi_token}).text
    return json.loads(json_string)['data']


def upload_image(file_path: str, plt_type: str = ''):
    """
    上传图片或Markdown中所有的图片到多平台（免API KEY，但不保证数据安全）

    Upload images or all images from Markdown to multiple platforms (API-free KEY, but data security is not guaranteed)

    :param file_path: 图片或Markdown文件路径
    :param plt_type: 平台（使用 qs -upload_image -help查看支持的平台）
    :param set_url: {v1_url} or {v2_url} 默认 v1
    :return: None
    """
    from .. import qs_default_console, qs_error_string, qs_info_string, qs_warning_string
    from ..TuiTools.Table import qs_default_table

    set_url = v2_url
    res_table = qs_default_table([
        {'header': "File" if user_lang != 'zh' else '文件', 'no_wrap': True, 'justify': "center", 'style': "bold cyan"},
        {'header': "Status" if user_lang != 'zh' else '状态', 'no_wrap': True, 'justify': "center"},
        {'header': "Url" if user_lang != 'zh' else '链接', 'no_wrap': True, 'justify': "center"},
    ])

    def post_img(path):
        if not os.path.exists(path):
            return False
        try:
            data = {'type': plt_type}
            file = [('image', open(path, 'rb'))]
        except:
            return False
        if plt_type:
            res_json = requests.post(set_url + 'image', data=data, files=file).text \
                if not alapi_token else \
                requests.post(set_url + 'image', data=data, files=file, headers={'token': alapi_token}).text
        else:
            res_json = requests.post(set_url + 'image', files=file).text \
                if not alapi_token else \
                requests.post(set_url + 'image', files=file, headers={'token': alapi_token}).text
        try:
            return json.loads(res_json)
        except Exception as e:
            qs_default_console.print(qs_error_string, repr(e), res_json)
            return False

    def get_path(rt, rel):
        return os.path.abspath(rt + rel)

    def format_markdown(path):
        import re
        from rich.progress import Progress
        from .. import user_root
        rt_path = dir_char.join(os.path.abspath(path).split(dir_char)[:-1]) + dir_char
        img_dict = {}
        with open(path, 'r') as fp:
            ct = fp.read()
        aims = re.findall('!\[.*?]\((.*?)\)', ct, re.M) + re.findall('<img.*?src="(.*?)".*?>', ct, re.M)
        progress = Progress(console=qs_default_console)
        pid = progress.add_task('  Upload' if user_lang != 'zh' else '  上传', total=len(aims))
        progress.start()
        progress.start_task(pid)
        for aim in aims:
            if aim.startswith('http://') or aim.startswith('https://'):  # Uploaded
                qs_default_console.print(qs_warning_string, aim,
                                         'is not a local file' if user_lang != 'zh' else '非本地文件')
                progress.advance(pid, 1)
                continue
            raw_path = aim
            aim = aim.replace('~', user_root)
            aim = aim if aim.startswith(dir_char) else get_path(rt_path, aim)
            if aim not in img_dict:
                qs_default_console.print(qs_info_string, 'Start uploading:' if user_lang != 'zh' else '正在上传:', aim)
                res_dict = post_img(aim)
                if not res_dict:
                    res_table.add_row(aim.split(dir_char)[-1], 'No File' if user_lang != 'zh' else '无文件', '')
                    img_dict[aim] = False
                else:
                    try:
                        res_table.add_row(
                            aim.split(dir_char)[-1], str(res_dict['code']),
                            res_dict['msg'] if res_dict['code'] != 200 else (
                                res_dict['data']['url']
                                if res_dict['data']['url'] else plt_type + ' failed')
                        )
                        img_dict[aim] = res_dict['data']['url'] if res_dict['code'] == 200 else False
                    except Exception:
                        qs_default_console.print(qs_error_string, res_dict)
                        res_table.add_row(aim.split(dir_char)[-1], str(res_dict['code']), res_dict['msg'])
                        img_dict[aim] = False

            if aim in img_dict and img_dict[aim]:
                qs_default_console.print(qs_info_string, 'replacing img:' if user_lang != 'zh' else '替换路径',
                                         f'"{raw_path}" with "{img_dict[aim]}"')
                ct = ct.replace(raw_path, img_dict[aim])
            progress.advance(pid, 1)
        progress.stop()
        with open(path, 'w') as fp:
            fp.write(ct)
        qs_default_console.print(res_table, justify="center")

    try:
        is_md = file_path.endswith('.md')
    except IndexError:
        qs_default_console.print(
            qs_error_string, 'Usage: qs -upimg {*.md} | {picture}'
            if user_lang != 'zh' else '使用: qs -upimg {Markdown文件} 或 {任意图片文件}')
    else:
        if is_md:
            format_markdown(file_path)
        else:
            res = post_img(file_path)
            if not res:
                res_table.add_row(os.path.basename(file_path), 'No File', '')
            else:
                try:
                    res_table.add_row(
                        os.path.basename(file_path), str(res['code']),
                        res['msg'] if res['code'] != 200 else res['data']['url']
                    )
                except Exception:
                    res_table.add_row(os.path.basename(file_path), str(res['code']), res['msg'])
            qs_default_console.print(res_table, justify="center")


def translate(text: str, from_lang: str = 'auto', to_lang: str = user_lang):
    """
    获取翻译结果

    Get the translation results.
    
    :param text: 待翻译内容 | Content to be translated.
    :param from_lang: 语种来源 | Source language
    :param to_lang: 翻译成的语种 | Translated into the language
    :return: 翻译的文本 | Translated text
    """
    global alapi_token
    try:
        request_info = 'q={}&from={}&to={}'.format(quote(text, 'utf-8'), from_lang, to_lang) if from_lang \
            else 'q={}&to={}'.format(quote(text, 'utf-8'), to_lang)
        res = requests.post(v2_url + 'fanyi', data=request_info,
                            headers={'Content-Type': "application/x-www-form-urlencoded", 'token': alapi_token}
                            if alapi_token else {'Content-Type': 'application/x-www-form-urlencoded'})
        if res.status_code == requests.codes.ok:
            res = json.loads(res.text)
            if res['code'] != 200:
                return "[ERROR] {}".format(res['msg'])
            return res['data']['dst']
        return "[ERROR] 未知错误 | Unknown Error"
    except Exception as unknown_error:
        return f"[ERROR] {repr(unknown_error)}"


def bili_cover(url: str):
    """
    获取BiliBili视频封面

    Get the BiliBili video cover

    :param url: BiliBili视频链接或视频号 | BiliBili video link or video number
    :return:
    """
    import re
    from ..NetTools.NormalDL import normal_dl
    from .. import qs_default_console, qs_error_string, qs_info_string
    from ..ImageTools.ImagePreview import image_preview
    try:
        res = requests.post(v2_url + 'bilibili/cover', data='c=' + url,
                            headers={'Content-Type': 'application/x-www-form-urlencoded', 'token': alapi_token})
    except Exception as e:
        return qs_default_console.print(qs_error_string, repr(e))
    if res.status_code == requests.codes.ok:
        res = json.loads(res.text)
        if res['code'] != 200 or res['msg'] != 'success':
            qs_default_console.log(
                qs_error_string, ("Get cover with: %s failed:" if user_lang != 'zh' else '下载封面: %s 失败: ') % url)
            return
        res = res['data']
        res['description'] = res['description'].replace('<br />', '\n\t')
        res['description'] = res['description'].replace('&nbsp;', ' ')
        res['description'] = re.sub('<.*?>', '', res['description']).strip()  # 忽略HTML标签
        qs_default_console.print(qs_info_string, 'TITLE:' if user_lang != 'zh' else '标题:', '%s' % res['title'])
        qs_default_console.print(qs_info_string, '' if user_lang != 'zh' else '简介:', end='\n\t')
        qs_default_console.print(res['description'], end='\n\n')
        normal_dl(res['cover'], res['title'] + '.' + res['cover'].split('.')[-1])
        image_preview(res['title'] + '.' + res['cover'].split('.')[-1])
    else:
        qs_default_console.log(qs_error_string,
                               f"Get cover with: {url} failed" if user_lang != 'zh' else f'下载封面: {url} 失败')


def ip_info(ip: str):
    """
    获取ip的运营商、地理位置等数据

    Get IP operator, geographic location and other data

    :param ip: ipv4, ipv6, empty means current machine
    :return: data dict {ip, isp, pos | ERROR MESSAGE, location | ERROR code}
    """
    try:
        res = requests.get(v2_url + ('ip?ip=%s&format=json' % ip if ip else 'ip') , headers={'token': alapi_token})
        res = json.loads(res.text)
        if res['code'] != 200:
            return {'ip': ip, 'isp': '', 'pos': res['msg'], 'location': '{code, %s}' % res['code']}
        return res['data']
    except Exception as e:
        return {'ip': ip, 'isp': '', 'pos': 'Network Error'
                if user_lang != 'zh' else '网络错误', 'location': '{code, %s}' % repr(e)}


def garbage_classification(query_ls: list):
    """
    查询中国垃圾分类

    Search Chinese garbage classification

    :param query_ls: 待查询的垃圾列表 | garbage list
    :return: 查询结果的字符串表格 | string table
    """
    from .. import qs_default_console, cut_string
    width = qs_default_console.width // 5 * 2 - 1

    def fmt_string(string, pre_num):
        return '\n' * pre_num + ' '.join(cut_string(string, width))

    from ..TuiTools.Table import qs_default_table
    from rich.text import Text
    import math

    table = qs_default_table([
        {'header': '名称', 'justify': 'center', 'style': 'bold cyan'},
        {'header': '分类', 'justify': 'center'},
        {'header': '解释', 'justify': 'center'},
        {'header': '提示', 'justify': 'center'},
    ], '[bold underline] 查询结果\n')
    first_flag = True
    for query_el in query_ls:
        if first_flag:
            first_flag = False
        else:
            table.add_row(Text('-', style='none'), Text('-', style='none'), '-', '-')
        res = requests.post(v2_url + 'garbage',
                            data="name={}".format(quote(query_el, 'utf-8')),
                            headers={'Content-Type': "application/x-www-form-urlencoded", 'token': alapi_token}
                            if alapi_token else {'Content-Type': 'application/x-www-form-urlencoded'})
        if res.status_code == requests.codes.ok:
            res = json.loads(res.text)
            if res['code'] != 200:
                table.add_row(fmt_string(query_el, 0), '', '', fmt_string(res['msg'], 0))
                continue
            try:
                res = res['data'][0]
                half_rows = [math.ceil(len(query_el) / width / 2), 1, math.ceil(len(res['explain']) / width / 2),
                             math.ceil(len(res['tip']) / width / 2)]
                max_row = max(half_rows)
                half_rows = [max_row - i for i in half_rows]
                table.add_row(
                    fmt_string(query_el, half_rows[0]),
                    fmt_string(
                        [
                            '[bold green]可回收', '[bold red]有害',
                            '[bold yellow]厨余(湿)', '[bold blue]其他(干)'
                        ][int(res['type']) - 1], half_rows[1]
                    ),
                    Text(fmt_string(res['explain'], half_rows[2]), justify='full'),
                    Text(fmt_string(res['tip'], half_rows[3]), justify='full')
                )
            except IndexError:
                table.add_row(fmt_string(query_el, 0), 'Unknown', '未知垃圾', '未知垃圾')
        else:
            table.add_row(fmt_string(query_el, 0), 'Unknown', 'None', 'Request Error')
    return table


def short_video_info(url: str):
    """
    解析多平台短视频，并返回视频信息与下载直链

    Parse multi - platform short video and return video information with download straight link

    :param url: 短视频分享链接 | Short video sharing link
    :return:
    """
    try:
        res = requests.post(v2_url + "video/url", data="url={}".format(quote(url, 'utf-8')),
                            headers={'Content-Type': "application/x-www-form-urlencoded", 'token': alapi_token}
                            if alapi_token else {'Content-Type': "application/x-www-form-urlencoded"})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, {'title': res['msg'], 'cover_url': 'None', 'video_url': 'None', 'source': str(res['code'])}
        return True, res['data']
    except Exception as e:
        return False, {'title': '网络错误' if user_lang == 'zh' else 'Network Error',
                       'cover_url': 'None', 'video_url': 'None', 'source': 'Unknown'}


def acg():
    """
    随机获取一张acg图片链接

    Get a random link to an ACG image

    :return: acg image link
    """
    try:
        res = requests.post(v2_url + 'acg', data='format=json',
                            headers={'Content-Type': "application/x-www-form-urlencoded", 'token': alapi_token}
                            if alapi_token else {'Content-Type': "application/x-www-form-urlencoded"})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, res['msg'], 0, 0
        return True, res['data']['url'], res['data']['width'], res['data']['height']
    except Exception as e:
        return False, repr(e), 0, 0


def bingImg():
    """
    随机获取一张bing图片链接

    Get a link to a bing picture at random

    :return: image link
    """
    try:
        res = requests.post(v2_url + 'bing', data='format=json',
                            headers={'Content-Type': "application/x-www-form-urlencoded", 'token': alapi_token}
                            if alapi_token else {'Content-Type': "application/x-www-form-urlencoded"})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, res['msg'], ''
        return True, res['data']['url'], res['data']['copyright']
    except Exception as e:
        return False, repr(e), ''


def kdCheck(kd_number: str):
    """
    查询中国快递

    Inquire about China Express

    :param kd_number: 快递单号 | courier number
    :return: bool, list
    """
    try:
        res = requests.post(v2_url + 'kd', data="number=%s" % kd_number,
                            headers={'Content-Type': 'application/x-www-form-urlencoded', 'token': alapi_token})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, 0, res['msg']
        return True, (int(res['data']['status']) if 'status' in res['data'] else int(res['data']['state'])), \
               res['data']['info']
    except Exception as e:
        return False, 0, repr(e)


def pinyin(zh_str: str):
    """
    查询中文的拼音

    Query Chinese pinyin

    :param zh_str: Chinese | 中文
    :return:
    """
    try:
        res = requests.post(v2_url + 'pinyin', data=f'word={quote(zh_str, "utf-8")}&tone=1&abbr=0',
                            headers={'Content-Type': 'application/x-www-form-urlencoded', 'token': alapi_token})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, res['msg']
        return True, res['data']['pinyin']
    except Exception as e:
        return False, repr(e)


def exchange(fr, number, to=user_currency):
    """
    查询<number> <fr> 对应 多少 <to>

    Query <number> <fr> corresponding <number?> <to>

    :param to: 待查询的币种
    :param number: 待查询的货币数量
    :param fr: 目标币种
    :return:
    """

    try:
        res = requests.get(v2_url + 'exchange',
                           params={'money': number, 'from': fr, 'to': to},
                           headers={'Content-Type': 'application/x-www-form-urlencoded', 'token': alapi_token})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, res['msg']
        return True, res['data']
    except Exception as e:
        return False, repr(e)


def zhihuDaily():
    """
    获取知乎日报

    Get Zhihu Daily

    :return:
    """
    try:
        res = requests.get(v2_url + 'zhihu',
                           headers={'Content-Type': 'application/x-www-form-urlencoded', 'token': alapi_token})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, res['msg']
        return True, res['data']
    except Exception as e:
        return False, repr(e)


def daily60s():
    """
    获取每日60秒早报

    Get the daily 60-second morning report

    :return:
    """
    try:
        res = requests.get(v2_url + 'zaobao?format=json', headers={'token': alapi_token})
        res = json.loads(res.text)

        if res['code'] != 200:
            return False, res['msg']
        return True, res['data']
    except Exception as e:
        return False, repr(e)


def doutu(keyword: str):
    """
    获取keyword的表情包

    :param keyword: 关键词
    :return: 图片链接
    """
    try:
        res = requests.post(v2_url + 'doutu',
                            params={'keyword': keyword},
                            headers={'Content-Type': 'application/x-www-form-urlencoded', 'token': alapi_token})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, res['msg']
        return True, res['data']
    except Exception as e:
        return False, repr(e)


def joke():
    """
    获取随机笑话

    :return: title, content
    """
    try:
        res = requests.get(v2_url + 'joke/random', headers={'token': alapi_token})
        res = json.loads(res.text)
        if res['code'] != 200:
            return False, res['msg']
        return True, res['data']
    except Exception as e:
        return False, repr(e)



from .. import headers, user_lang
from . import pre_check
import requests
import json

lolicon_token = pre_check('lolicon_token', False)


def loli_img(keyword: str = ''):
    try:
        res = requests.get(
            'https://api.lolicon.app/setu/', headers=headers,
            params={
                'apikey': lolicon_token if lolicon_token else '',
                'r18': '2',
                'keyword': keyword,
                'num': 1,
                'proxy': 'disable'
            }
        )
    except Exception as e:
        return False, repr(e), False
    else:
        if res.status_code != requests.codes.ok:
            return False, ("Get Data Failed" if user_lang != 'zh' else "获取图源数据失败"), False
        data = json.loads(res.text)
        return data['code'] == 0, data['msg'], data['data']


def magnet2torrent(hash: str):
    try:
        from ..NetTools import get_fileinfo
        from ..NetTools.NormalDL import normal_dl
        infos = get_fileinfo('https://m2t.lolicon.app/m/' + hash)
        normal_dl(infos[0], set_name=f'{infos[-1].headers["torrent-name"]}.torrent')
        return True, 'Success'
    except Exception as e:
        return False, repr(e)



# coding=utf-8
"""
qs的WIFI模块, 协助连接WIFI, 但不支持机构WIFI自动登录

The WiFi module of QS helps to connect WiFi, but does not support the automatic login of institutional WiFi
"""
import time
from .. import user_lang, qs_default_console, qs_error_string, requirePackage

const = requirePackage('pywifi', 'const')
PyWiFi = requirePackage('pywifi', 'PyWiFi')


class WiFi:
    def __init__(self) -> None:
        """
        qs的wifi工具 (Windows 或 Linux)
        """
        self.wifi_engine = PyWiFi()
        self.ifaces = self.wifi_engine.interfaces()
        self.iface = self.ifaces[0]

    def status(self) -> str:
        """
        判断当前wifi连接状态

        Determine the current wifi connection status

        :return: 连接的wifi名 | The name of the connecting wifi
        """
        flag = self.iface.status() in [const.IFACE_CONNECTED, const.IFACE_CONNECTING]
        return self.iface.name if flag else ''

    def scan(self):
        """
        扫描附近可连接的wifi

        Scan for nearby wifi connections

        :return: 按信号强度排序好的可连接wifi列表 | A list of available wifi connections sorted by signal strength
        """
        self.iface.scan()
        time.sleep(1)
        res = self.iface.scan_results()
        wifi_ls = []
        for i in res:
            if i not in wifi_ls and i.signal > -90:
                wifi_ls.append((i.ssid, i.signal, i.akm))
        return sorted(wifi_ls, key=lambda x: x[1], reverse=True)

    def set_iface(self):
        """
        设置网卡接口

        Set Network port

        :return: None
        """
        num = len(self.ifaces)
        if num <= 0:
            qs_default_console.log(
                qs_error_string, "There is no recognizable network card interface"
                if user_lang != 'zh' else '没有可识别的网卡接口')
            return
        elif num != 1:
            from ..TuiTools.Table import qs_default_table
            table = qs_default_table(['id', 'interface'] if user_lang != 'zh' else ['序号', '网卡'])
            for i, w in enumerate(self.ifaces):
                table.add_row(str(i), w.name())
            qs_default_console.print(table)
            while True:
                iface_no = input('请选择网卡接口序号: ' if user_lang != 'zh' else 'Select interface by id: ')
                try:
                    no = int(iface_no)
                    if 0 <= no < num:
                        self.iface = self.ifaces[no]
                        break
                except:
                    continue
        else:
            qs_default_console.print(f"{'Only one' if user_lang != 'zh' else '仅有可用'}: {self.iface.name()}")

    def conn(self, ssid, password):
        """
        连接WiFi

        connect WiFi

        :param ssid: wifi名称
        :param password: 密码
        :return: status
        """
        # from pywifi import Profile
        Profile = requirePackage('pywifi', 'Profile')
        self.iface.disconnect()
        time.sleep(1)
        pinfo = Profile()
        pinfo.ssid = ssid
        pinfo.auth = const.AUTH_ALG_OPEN
        pinfo.akm.append(const.AKM_TYPE_WPA2PSK)
        pinfo.cipher = const.CIPHER_TYPE_CCMP
        pinfo.key = password
        self.iface.remove_all_network_profile(pinfo)
        tmp = self.iface.add_network_profile(pinfo)
        self.iface.connect(tmp)
        time.sleep(5)
        return self.status()

    def disconn(self):
        """
        断开wifi连接

        disconnect wifi
        :return: status
        """
        self.iface.disconnect()
        return self.status()



# coding=utf-8
"""
qs的普通文件下载器！干翻迅雷，并且支持设置代理，设置referer，好看的命令行交互，啥j8文件都能下；(仅支持http[s]协议)

QS ordinary file downloader! Dry out Thunder, and support to set the proxy, set the Referer,
good-looking command line interaction, and it can download fuck any files; (only supports HTTP[S])

Author: RhythmLian (https://rhythmlian.cn)
"""
from . import size_format, get_fileinfo
from concurrent.futures import ThreadPoolExecutor, wait
from ..ThreadTools import FileWriters
from ..SystemTools import get_core_num
from .. import user_lang, qs_default_console, qs_error_string, qs_info_string, qs_warning_string, headers
from threading import Lock
from requests import get
import signal
import queue
import time
import sys
import os

core_num = get_core_num()
maxBlockSize = int(5e6)
minBlockSize = int(5e5)


def GetBlockSize(sz):
    """
    自动规划块大小

    Automatically program the block size

    :param sz: 文件大小
    :return: 块大小
    """
    if sz >= maxBlockSize << 10:
        return maxBlockSize
    elif sz <= minBlockSize << 3:
        return minBlockSize
    else:
        return max(sz >> 9, minBlockSize)


def ask_overwrite(path: str):
    from ..__config__ import prompt

    return prompt({
        'type': 'confirm',
        'name': 'overwrite',
        'message': f'"{path}" 已存在, 是否覆盖?' if user_lang == 'zh' else f'"{path}" already exists, overwrite?',
        'default': False
    })['overwrite']


class Downloader:
    from ..TuiTools.Bar import DataTransformBar

    def __init__(self, url: str, num: int, name: str = '', proxy: str = '',
                 referer: str = '', output_error: bool = False, failed2exit: bool = False, exit_if_exist: bool = False,
                 disableStatus: bool = False, disableParallel: bool = False):
        """
        qs普通文件下载引擎

        Qs general file download engine

        :param url: 文件url
        :param num: 线程数量
        """
        signal.signal(signal.SIGINT, self._kill_self)
        info_flag = True
        self.url, self.num, self.output_error, self.proxies = url, num, output_error, {}
        self.exit_if_exist = exit_if_exist
        self.enabled = True
        if not disableStatus:
            with qs_default_console.status('Getting file info..' if user_lang != 'zh' else '获取文件信息中..'):
                self.url, self.name, r = get_fileinfo(url, proxy, referer)
        else:
            self.url, self.name, r = get_fileinfo(url, proxy, referer)
        if not (self.url and self.name and r):
            info_flag = False
            qs_default_console.print(qs_error_string if failed2exit else qs_warning_string,
                                     'Get File information failed, please check network!'
                                     if user_lang != 'zh' else '获取文件信息失败，请检查网络!')
            if failed2exit:
                self.enabled = False
                return
        if self.name and '.' not in self.name:
            self.name = os.path.basename(url)
        if name:
            self.name = name

        if os.path.exists(self.name) and self.exit_if_exist:
            self.enabled = False
            return

        if proxy:
            self.proxies = {
                'http': 'http://'+proxy,
                'https': 'https://'+proxy
            }
        self.headers = headers
        if referer:
            self.headers['Referer'] = referer
        if not self.url:
            qs_default_console.print(qs_error_string, self.name)
            raise Exception('Connection Error!' if user_lang != 'zh' else '连接失败!')
        try:
            if not info_flag:
                raise KeyError
            self.size = int(r.headers['content-length'])
            self.main_progress = Downloader.DataTransformBar()
            self.dl_id = self.main_progress.add_task('Download', filename=self.name, start=False)
            self.main_progress.update(self.dl_id, total=self.size)
            if self.size < 5e6 or disableParallel:
                qs_default_console.print(qs_info_string, 'FILE SIZE' if user_lang != 'zh' else '文件大小'
                                         , size_format(self.size))
                self.size = -self.size
            else:
                self.fileBlock = GetBlockSize(self.size)
        except KeyError:
            self.size = -1
            self.main_progress = Downloader.DataTransformBar(False)
            self.dl_id = self.main_progress.add_task('Download', filename=self.name, start=False)
        if self.size > 0:
            self.pool = ThreadPoolExecutor(max_workers=self.num)
            self.futures, self.fileLock = [], Lock()
            self.job_queue = queue.Queue()
            if os.path.exists(self.name + '.qs_dl'):
                self.ctn_file = open(self.name + '.qs_dl', 'r+')
                self.ctn = [int(i) for i in self.ctn_file.read().strip().split()]
            else:
                self.ctn_file = open(self.name + '.qs_dl', 'w')
                self.ctn = []
            self.writers = FileWriters(self.name, max(2, int(core_num / 2)), "rb+" if self.ctn else "wb")
            qs_default_console.print(qs_info_string, 'FILE  SIZE' if user_lang != 'zh' else '文件大小'
                                     , size_format(self.size, align=True))
            qs_default_console.print(qs_info_string, 'THRAED NUM' if user_lang != 'zh' else '线程数量'
                                     , '%7d' % num)

    def _kill_self(self, signum, frame):
        """
        终止下载任务，保存断点

        Terminate the download and save the breakpoint

        :param signum: 信号
        :param frame: frame
        :return: None
        """
        if self.size > 0:
            self.main_progress.stop_task(self.dl_id)
            self.main_progress.stop()
            qs_default_console.print(qs_info_string,
                                     'Get Ctrl-C, exiting...' if user_lang != 'zh' else '捕获Ctrl-C, 正在退出...')
            self.ctn_file.close()
            self.pool.shutdown(wait=False)
            self.writers.wait()
            qs_default_console.print(qs_info_string, 'Deal Done!' if user_lang != 'zh' else '处理完成!')
        os._exit(0)

    def _dl(self, start):
        """
        执行文件块下载任务

        Perform the file block download task

        :param start: 文件块起始偏移量
        :return: None
        """
        try:
            _sz = min(start + self.fileBlock, self.size - 1)
            _headers = self.headers.copy()
            _headers['Range'] = 'bytes={}-{}'.format(start, _sz)
            _content = b''
            for chunk in get(self.url, headers=_headers, timeout=50, proxies=self.proxies).iter_content(65536):
                _content += chunk
                self.main_progress.advance(self.dl_id, sys.getsizeof(chunk))
            self.writers.new_job(_content, start)
        except Exception as e:
            if self.output_error:
                qs_default_console.print(qs_error_string, repr(e))
            self.job_queue.put(start)
        else:
            self.fileLock.acquire()
            self.ctn.append(start)
            self.ctn_file.write('%d\n' % start)
            self.fileLock.release()

    def _single_dl(self):
        """
        无法并行下载或无需并行下载时采用串行下载

        Parallel downloads are not possible or serial downloads are not required

        :return: None
        """
        r = get(self.url, stream=True, proxies=self.proxies, headers=self.headers)
        flag = self.size != -1

        if flag:
            self.main_progress.start_task(self.dl_id)
        else:
            self.main_progress.update(self.dl_id, total=-1)
        with open(self.name, 'wb') as f:
            for chunk in r.iter_content(32768):
                f.write(chunk)
                self.main_progress.advance(self.dl_id, sys.getsizeof(chunk))

    def run(self):
        """
        规划下载任务并开始下载

        qs可以有效应对网络问题对下载任务造成的影响，如：
          1.下载过程中切换代理

          2.链路异常
        并确保下载任务顺利完成

        :return: None
        """
        if not self.enabled:
            return self.name if self.exit_if_exist else ''
        self.main_progress.start()
        if self.size > 0:
            self.main_progress.start_task(self.dl_id)
            if not self.ctn:
                with open(self.name, "wb") as fp:
                    fp.truncate(self.size)
            self.main_progress.advance(self.dl_id, self.fileBlock*len(self.ctn))
            for i in range(0, self.size, self.fileBlock):
                if self.ctn and i in self.ctn:
                    continue
                else:
                    self.job_queue.put(i)
            retry_cnt = 0
            while not self.job_queue.empty():
                self.futures.clear()
                while not self.job_queue.empty():
                    cur = self.job_queue.get()
                    if cur not in self.ctn:
                        self.futures.append(self.pool.submit(self._dl, cur))
                wait(self.futures)
                if not self.job_queue.empty() and retry_cnt > 2:
                    qs_default_console.print(qs_warning_string, 'Exists File Block Lost, Retrying after 0.5 sec'
                                             if user_lang != 'zh' else '存在文件块丢失，0.5秒后重试')
                    time.sleep(0.5)
                retry_cnt += 1
            self.writers.wait()
            self.ctn_file.close()
            os.remove(self.name + '.qs_dl')
        else:
            self._single_dl()
        self.main_progress.stop()
        qs_default_console.print(qs_info_string, self.name, 'download done!' if user_lang != 'zh' else '下载完成!')
        return self.name


def normal_dl(url, set_name: str = '', set_proxy: str = '', set_referer: str = '',
              thread_num: int = min(16, core_num * 4), output_error: bool = False, failed2exit: bool = False, exit_if_exist: bool = False,
              disableStatus: bool = False, disableParallel: bool = False):
    """
    自动规划下载线程数量并开始并行下载

    Automatically schedule the number of download threads and begin parallel downloads

    :param disableStatus:
    :param url: 文件url
    :param set_name: 设置文件名（默认采用url所指向的资源名）
    :param set_proxy: 设置代理（默认无代理）
    :param set_referer: 设置referer
    :param thread_num: 线程数
    :param output_error: 输出报错信息
    :param failed2exit: 获取文件信息失败则不下载，否则qs将继续尝试下载
    :param disableStatus: 是否显示当前任务状态
    :return: file name
    """
    if set_name and os.path.exists(set_name) and not ask_overwrite(set_name):
        return ''
    return Downloader(
        url=url, num=thread_num, name=set_name, proxy=set_proxy,
        referer=set_referer, output_error=output_error, failed2exit=failed2exit, exit_if_exist=exit_if_exist,
        disableStatus=disableStatus, disableParallel=disableParallel
    ).run()



# coding=utf-8
"""
系统工具库

System tool library
"""


def clear_mem():
    """
    清理系统内存

    Clean system memory

    :return: None
    """
    from .. import dir_char, system

    if dir_char == '\\':
        print("Not support")
    else:
        import os
        if system.startswith("darwin"):
            os.system("sudo purge")
        else:
            os.system('sync')
            os.system("echo 3 > /proc/sys/vm/drop_caches")


def get_core_num():
    """
    获取核数量

    :return: core num
    """
    from .. import requirePackage
    return requirePackage('psutil').cpu_count()



# coding=utf-8
"""
qs的API模块, 使用前确保配置文件~/.qsrc中已经配置相关的KEY值

Before using the API module of QS, make sure that the relevant key value has been configured in the configuration file ~/.qsrc
"""
from .. import qs_config, user_lang, user_root, dir_char, system, headers


def pre_check(funcName: str, ext: bool = True) -> str:
    """
    获取用户保存的API KEY

    Gets the API KEY saved by the user.

    :param funcName: API KEY在~/.qsrc的名称 | API KEY in the name of the .qsrc.
    :param ext: 获取失败是否退出程序 | Get failed whether to exit the program

    :return: 找到的API KEY | API KEY found.
    """
    try:
        api_key = qs_config.apiSelect(funcName)
        if api_key.startswith("GET:"):
            raise KeyError
    except KeyError:
        if ext:
            exit('You should set %s api key at: %s.qsrc' % (funcName, user_root)
                 if user_lang != 'zh' else '你需要在qs的配置表 %s.qsrc 中填入设%s键值' % (user_root, funcName))
        else:
            return ''
    else:
        return api_key
